package org.playonline.network.datatransmission;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;

import org.playonline.network.client.UGASPClient;
import org.playonline.network.datatransmission.events.EventsMgr;
import org.playonline.network.datatransmission.protocol.rmai.RMAIMgr;

public interface DataTransmissionMgr {
	
	UGASPClient 			getUGASPClient();
	RMAIMgr 				getRMAIMgr();
	EventsMgr 				getEventsMgr();
	DataOutputStream 		getStorageBufferOutputStream();
	ByteArrayOutputStream 	getStorageBufferByteArrayOutputStream();
	void					reinitializeStorageBuffer();
	void					resetStorageBuffer();
	void					setDisconnected(boolean disconnected);
	
	/** 
	 *  This method represents the routine for client when 
	 *  a game instance is started : 
	 *  <ul>
	 *  <li>client send RMAI if there are some</li>
	 *  <li>client is getting events from platform if there are some</li>
	 *  <li>then, it sleeps for a time interval corresponding to GASPClient.period.</li>
	 *  </ul>
	 *  Method stops as soon as :
	 *  <ul>
	 *  <li>QuitApplicationInstance, EndApplicationInstance or QuitPlatform have been called on the LobbyMgr of the uGASP Client</li>
	 *  <li>a disconnection occurs.</li>
	 *  </ul> 
	 */
	void 					launchDataTransmissionRoutine();
}
