package org.playonline.network.datatransmission.protocol.exception;

/**
 * This Exception is raised when the highest version of the data transmission protocol
 * handled by uGASP platform is lower than the one used by the client.
 * 
 * @author Gabriel Adgeg (gabriel.adgeg@telecom-sudparis.eu)
 */
public class DataTransmissionProtocolVersionException extends Exception {
	
	private byte clientDataTransmissionProtocolVersion;
	private byte platformDataTransmissionProtocolVersion;
	
	public DataTransmissionProtocolVersionException(byte clientDataTransmissionProtocolVersion, byte platformDataTransmissionProtocolVersion){
		this.clientDataTransmissionProtocolVersion 		= clientDataTransmissionProtocolVersion;
		this.platformDataTransmissionProtocolVersion 	= platformDataTransmissionProtocolVersion;
	}
	
	public void printMessage(){
		System.err.println("Client version of Data Transmission Protocol ("+clientDataTransmissionProtocolVersion+") is higher than platform one ("+
				platformDataTransmissionProtocolVersion+")." +
				"Please download a more recent version of uGASP platform.");
	}
	
}
