package org.playonline.network.datatransmission.protocol.rmai;

import org.playonline.network.datatransmission.protocol.rmai.tools.RMAIUtils;

public interface RMAIMgr {

	RMAIUtils 	getRmaiUtils();
	
	/**
	 * Invoke asynchronously a remote method without parameters either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void 		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName);
	
	/**
	 * Invoke asynchronously a remote method either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * Here are the types that can be used as parameter of this remote method:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Short</li>
	 * <li>Integer</li>
	 * <li>Long</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>String</li>
	 * <li>RMAISerializable (you should pay a special attention on recursion : never pass this object reference).</li>
	 * <li>Arrays of those several types</li>
	 * </ul>
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void 		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName, Object param0);
	
	/**
	 * Invoke asynchronously a remote method either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * Here are the types that can be used as parameter of this remote method:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Short</li>
	 * <li>Integer</li>
	 * <li>Long</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>String</li>
	 * <li>RMAISerializable (you should pay a special attention on recursion : never pass this object reference).</li>
	 * <li>Arrays of those several types</li>
	 * </ul>
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @param param1					second parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void 		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName, Object param0,Object param1);
	
	/**
	 * Invoke asynchronously a remote method either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * Here are the types that can be used as parameters of this remote method:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Short</li>
	 * <li>Integer</li>
	 * <li>Long</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>String</li>
	 * <li>RMAISerializable (you should pay a special attention on recursion : never pass this object reference).</li>
	 * <li>Arrays of those several types</li>
	 * </ul>
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @param param1					second parameter of the method that should be invoked
	 * @param param2					third parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void 		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName, Object param0,Object param1, Object param2);
	
	/**
	 * Invoke asynchronously a remote method without either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * Here are the types that can be used as parameters of this remote method:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Short</li>
	 * <li>Integer</li>
	 * <li>Long</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>String</li>
	 * <li>RMAISerializable (you should pay a special attention on recursion : never pass this object reference).</li>
	 * <li>Arrays of those several types</li>
	 * </ul>
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @param param1					second parameter of the method that should be invoked
	 * @param param2					third parameter of the method that should be invoked
	 * @param param3					fourth parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void 		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3);
	
	/**
	 * Invoke asynchronously a remote method either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * Here are the types that can be used as parameters of this remote method:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Short</li>
	 * <li>Integer</li>
	 * <li>Long</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>String</li>
	 * <li>RMAISerializable (you should pay a special attention on recursion : never pass this object reference).</li>
	 * <li>Arrays of those several types</li>
	 * </ul>
	 * 	
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @param param1					second parameter of the method that should be invoked
	 * @param param2					third parameter of the method that should be invoked
	 * @param param3					fourth parameter of the method that should be invoked
	 * @param param4					fifth parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3, Object param4);
	
	/**
	 * Invoke asynchronously a remote method either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * Here are the types that can be used as parameters of this remote method:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Short</li>
	 * <li>Integer</li>
	 * <li>Long</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>String</li>
	 * <li>RMAISerializable (you should pay a special attention on recursion : never pass this object reference).</li>
	 * <li>Arrays of those several types</li>
	 * </ul>
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @param param1					second parameter of the method that should be invoked
	 * @param param2					third parameter of the method that should be invoked
	 * @param param3					fourth parameter of the method that should be invoked
	 * @param param4					fifth parameter of the method that should be invoked
	 * @param param5					sixth parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void 		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3, Object param4, Object param5);
	
	/**
	 * Invoke asynchronously a remote method either :
	 * <ul>
	 * <li>on a uGASP Client, designated by its actor session id</li>
	 * <li>or on the uGASP game logic (RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients (RMAIClientManager.ActorSessionID.BROADCAST_WITHOUT_GAME_LOGIC.getID())</li>
	 * <li>or on every connected uGASP clients and uGASP game logic (RMAIClientManager.ActorSessionID.BROADCAST_WITH_GAME_LOGIC.getID())</li>
	 * </ul>
	 * Methods that should be invoked must be implemented in the class RMAI client manager.
	 * Only instances methods can be invoked.
	 * Here are the types that can be used as parameters of this remote method:
	 * <ul>
	 * <li>Boolean</li>
	 * <li>Byte</li>
	 * <li>Short</li>
	 * <li>Integer</li>
	 * <li>Long</li>
	 * <li>Double</li>
	 * <li>Float</li>
	 * <li>String</li>
	 * <li>RMAISerializable (you should pay a special attention on recursion : never pass this object reference).</li>

	 * <li>Arrays of those several types</li>
	 * </ul>
	 * 
	 * @param recipientActorSessionID	the actorSession ID of the suitable recipient
	 * @param methodName				the name of the method that should be invoked
	 * @param param0					first parameter of the method that should be invoked
	 * @param param1					second parameter of the method that should be invoked
	 * @param param2					third parameter of the method that should be invoked
	 * @param param3					fourth parameter of the method that should be invoked
	 * @param param4					fifth parameter of the method that should be invoked
	 * @param param5					sixth parameter of the method that should be invoked
	 * @param param6					seventh parameter of the method that should be invoked
	 * @return boolean					true if the response code of the HTTP connection is OK
	 * 									false otherwise
	 */
	void 		remoteMethodAsynchronousInvocation(short recipientClientActorSessionID, String methodName, Object param0, Object param1, Object param2, Object param3, Object param4, Object param5, Object param6);
}
