package org.playonline.network.datatransmission.protocol.rmai.tools;


import java.util.ArrayList;

/**
 * This class ensures a link between a String representation of a method name and
 * an Object[] representing parameters, to facilitate dynamic invocation of method. 
 *
 */
public class InvokedMethod {

	private String methodName;
	private Object[] parameters;
	
	
	public InvokedMethod(String methodName, ArrayList<Object> parameters){
		this.methodName = methodName;
		// if method has any argument
		if(parameters != null){
			this.parameters = parameters.toArray();
		}else{
			// otherwise
			this.parameters = null;
		}
	}
	
	
	public String getMethodName() {
		return methodName;
	}
	
	
	public Object[] getParameters() {
		return parameters;
	}
	
	/**
	 * Get an array of classes representing the types of this InvokedMethod parameters.
	 * This method is needed to provide the suitable arguments to the Method class, in order to invoke
	 * the required method.
	 * @return Class[]	a Class array matching with parameters (or null if parameters are null)
	 * 					Example: 
	 * 					Object[] parameters = {"hello",2,true} return
	 * 					Class[] types ={String,Integer,Boolean)
	 * 								
	 */
	public Class[] getParametersTypes() {
		// if there isn't any parameters
		if(parameters == null)return null;
		// otherwise
		Class[] parametersType = new Class[parameters.length];
		for (int i = 0; i < parametersType.length; i++) {
			// get the type of the parameters
			// method getPrimitiveTypeIfNeeded should be used
			parametersType[i] = parameters[i].getClass();
		}
		return parametersType;	
	}
	
	
	public String toString(){
		//method name
		String stringRepresentation = new String("Method name: "+this.methodName+"\n");
		for (int i = 0; i < parameters.length; i++) {
			stringRepresentation += "Parameter n"+i+" instance of "+parameters[i].getClass().toString()+", value = "+parameters[i]+"\n";
		}
		return stringRepresentation;
	}

}
