package org.playonline.network.datatransmission.protocol.rmai.tools;


import java.io.IOException;

import org.xmlpull.v1.XmlSerializer;

/**
 *  Class used to serialize an array of Object into an XML serializer.
 */
class RMAICommon {

	protected XmlSerializer xmlSerializer;
	protected RMAISerializer rmaiSerializer;
	
	RMAICommon() {
		xmlSerializer = new DefaultXMLSerializer();
		rmaiSerializer = new RMAISerializerImpl();
	}

	/**
	 * Sets custom RMAISerializer serializer (in case when server doesn't support
	 * standard XMLRPC protocol)
	 * 
	 * @param serializer custom serializer
	 */
	public void setRMAISerializer(RMAISerializer serializer) {
		rmaiSerializer = serializer;
	}
	
	
	XmlSerializer getXmlSerializer(){
		return xmlSerializer;
	}
	
	
	RMAISerializer getRMAISerializer(){
		return rmaiSerializer;
	}
			
	void serializeParams(Object[] params) throws Exception, IllegalArgumentException, IllegalStateException, IOException {
		if (params != null && params.length != 0)
		{
			// set method parameters
			xmlSerializer.startTag(null, Tag.PARAMS);
			for (int i=0; i<params.length; i++) {
				xmlSerializer.startTag(null, Tag.PARAM).startTag(null, RMAISerializer.TAG_VALUE);
				rmaiSerializer.serialize(xmlSerializer, params[i]);
				xmlSerializer.endTag(null, RMAISerializer.TAG_VALUE).endTag(null, Tag.PARAM);
			}
			xmlSerializer.endTag(null, Tag.PARAMS);
		}
	}

}
