package org.playonline.network.id;

public class ID {
	
	private static final short INVALID_ACTOR_SESSION_ID = -1;
	
	private String 	myUsername;
	private int 	myActorID;
	private int 	mySessionID;
	private short 	myActorSessionID;
	
	public ID(){
		myUsername = "";
	}
	
	/**
	 * Get the actorID of the GASPClient, i.e. the identifier of a client for a specific application.
	 * ActorID is created on the first login of a client on a specific application, then saved in uGASP database.
	 * 
	 *@return	the actorID, identifier of a client registered in uGASP database for a specific application
	 **/
	public int getActorID(){
		return myActorID;
	}


	/**
	 * Get the sessionID of the GASPClient, i.e. the identifier of a client connected to uGASP platform.
	 * A positive value for this sessionID means that the client is connected to uGASP. 
	 * A client MUST be registered into uGASP database via first login to be able to get a sessionID.
	 * 
	 *@return 	the sessionID, identifier of a client connected to uGASP platform
	 **/
	public int getSessionID(){
		return mySessionID;
	}


	/**
	 * Get the actorSessionID of the GASPClient, i.e. the identifier of a client connected to an application instance.
	 * A strictly positive value means that client has already called one of the following methods	
	 * with success: joinApplicationInstance(), joinRandomApplicationInstance(), createApplicationInstance(), 
	 * createPrivateApplicationInstance().
	 * 
	 * Otherwise, the value is 0.
	 * 
	 *@return int 	the actorSessionID
	 **/
	public short getActorSessionID(){
		return myActorSessionID;
	}

	/**
	 * Get the username of the GASPClient.
	 * 
	 *@return String		the username
	 **/
	public String getUsername(){
		return myUsername;
	}
	
	public void setActorID(int myActorID) {
		this.myActorID = myActorID;
	}
	
	public void setSessionID(int mySessionID) {
		this.mySessionID = mySessionID;
	}
	
	public void setActorSessionID(short myActorSessionID) {
		this.myActorSessionID = myActorSessionID;
	}
	
	/**
	 * Set the username of the GASPClient.
	 * 
	 *@param String		the username
	 **/
	public void setUsername(String username){
		myUsername=username;
	}
	
	/**
	 * Tell whether this actor session ID of the client is valid or not.
	 * This method should be used before trying to quit or to end an application instance.
	 * 
	 * @return 	true 	if actor session ID is strictly superior to 0,
	 * 			false	otherwise.
	 */
	public boolean isActorSessionIDValid(){
		return (myActorSessionID > 0);
	}
	
	/**
	 * Set the actor session ID to an invalid value.
	 * 
	 */
	public void invalidateActorSessionID(){
		this.myActorSessionID = INVALID_ACTOR_SESSION_ID;
	}
}
