package org.playonline.ui.basic;

import java.util.Locale;

import org.playonline.ui.basic.profile.InventoryActivity;
import org.playonline.ui.basic.profile.TransportActivity;

import android.app.TabActivity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager.NameNotFoundException;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SubMenu;
import android.view.View;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.TabHost.TabContentFactory;

public class PlayerProfileTabActivity extends TabActivity implements TabContentFactory{

	public final static int ENGLISH_ITEM_ID = 1001;
	public final static int FRENCH_ITEM_ID 	= 1002;
	
	private TabHost tabHost;
	private short 	availableTransportPoints;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// retrieve data from PlayOnlineMapActivity
		Bundle extras = getIntent().getExtras(); 
		if(extras != null){
			availableTransportPoints = extras.getShort(TransportActivity.POINTS_KEY);
		}else{
			availableTransportPoints = 0;
		}
		setContentView(R.layout.tab);

		tabHost = getTabHost();
		addTabs();
	}
	
	
	private void addTabs(){
		tabHost.addTab(tabHost.newTabSpec("tab1").setIndicator("Inventaire ").setContent(new Intent(this,InventoryActivity.class)));
		tabHost.addTab(tabHost.newTabSpec("tab2").setIndicator("Caractéristiques").setContent(this));
		// put data to intent
		Intent intent = new Intent(this,TransportActivity.class);
		intent.putExtra(TransportActivity.POINTS_KEY, availableTransportPoints);
		tabHost.addTab(tabHost.newTabSpec("tab3").setIndicator("Transport").setContent(intent));
	}


	/**
	 * Callback to make the tab contents
	 */
	public View createTabContent(String tag) {
		if(tag.compareTo("tab2")==0){
			TextView tv = new TextView(this);
			tv.setText("Mes caractéristiques");
			tv.setTextSize((float)24);
			return tv;
		} else if(tag.compareTo("tab3")==0){
			TextView tv = new TextView(this);
			tv.setText("Mes pouvoirs");
			tv.setTextSize((float)24);
			return tv;
		} else{
			return null;
		}
	}

	
	/****************************************************************************************************************/
	/************************************ 		Management of Locale       ******************************************/
	/****************************************************************************************************************/
	
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		SubMenu subMenuLanguage = menu.addSubMenu(0, 0, 0, this.getResources().getString(R.string.switch_language));
		subMenuLanguage.add(0,ENGLISH_ITEM_ID,0,"English");
		subMenuLanguage.add(0,FRENCH_ITEM_ID,0,"Français");
		return true;
	}


	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch(item.getItemId()){
		case ENGLISH_ITEM_ID:
			setLocal(Locale.ENGLISH);
			break;
		case FRENCH_ITEM_ID:
			setLocal(Locale.FRENCH);
			break;
		default:
			break;
		}
		return true;
	}


	private void setLocal(Locale locale){
		Resources res = getResources();
		Configuration configuration = res.getConfiguration();
		configuration.locale = locale;
		res.updateConfiguration(configuration, res.getDisplayMetrics());
		try {
			Context context = createPackageContext(this.getPackageName(), CONTEXT_INCLUDE_CODE);
			Intent intent = new Intent(context,PlayerProfileTabActivity.class);
			startActivity(intent);
			// finish current activity as it is in former language
			finish();
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
	}


}
