package org.playonline.ui.basic.db;

import org.playonline.network.client.http.UGASPClientHttp;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.widget.Toast;

public class DBAdapter {
	
	DatabaseHelper 	DBHelper;
	Context 		context;
	SQLiteDatabase 	db;
	
	public static DBAdapter getInstance(Context context){
		return new DBAdapter(context);
	}
	
	private DBAdapter(Context context){
		this.context = context;
		DBHelper = new DatabaseHelper(context);
	}
	
	public class DatabaseHelper extends SQLiteOpenHelper{

		Context 		context;
		
		public DatabaseHelper(Context context) {
			super(context, "attributes", null, 1);
			this.context = context;
		}
		
		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL("create table attributes (name text primary key, value text not null);");
			// default value for uGASP address
			ContentValues values = new ContentValues();
			values.put("name", "address");
			values.put("value", UGASPClientHttp.uGASPDefaultAddress);
			db.insert("attributes", null, values);
		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			Toast.makeText(context, "Database update from version "+oldVersion+" to version "+newVersion, Toast.LENGTH_LONG);
			db.execSQL("DROP TABLE IF EXISTS attributes");
			onCreate(db);
		}
		
	}
	
	public DBAdapter open(){
		db = DBHelper.getWritableDatabase();
		return this;
	}
	
	public void close() {
		db.close();
	}
	
	public void removeAll(){
		db.execSQL("DELETE FROM attributes");
	}
	
	public void putAddress(String value){
		ContentValues values = new ContentValues();
		values.put("value", value);
		db.update("attributes", values, "name='address'", null);
		//db.execSQL("UPDATE attributes SET value='"+value+"' where name='address';");
	}
	
	public Cursor getAttributes(){
		return db.query("attributes", new String[]{"name","value"}, null, null, null, null, null);
	}
}
