package org.playonline.ui.basic.dialog;

import java.util.List;
import java.util.StringTokenizer;

import org.playonline.ui.basic.dialog.handler.InGameDialogFrameMessage;

import android.os.Handler;
import android.os.Message;

public class FlashingTextView {
	
	
	private static final long 	FLASHING_DURATION 	= 150;//ms
	private final Handler handler;
	
	private FlashingTextView(Handler handler){
		this.handler = handler;
	}
	
	public static FlashingTextView getInstance(Handler handler){
		return new FlashingTextView(handler);
	}
	
	
	/**
	 * This method flash text in the dialog frame and send a message to DialogFrame's thread to set all views visible
	 * at the end of the flashing.
	 * 
	 * It is based on a separate thread that send message to the DialogFrame's one,
	 * to avoid freezing while calling Thread.sleep(FLASHING_DURATION) method.
	 *  
	 * @param text the text to flash.
	 */
	public void flashOtherCharacterText(CharSequence text){
		// used to parse the text
		final StringTokenizer tokenizer = new StringTokenizer((String)text);
		// special thread to flash the text
		Runnable runnable = new Runnable() {
			public void run() {
				flashTokens(tokenizer);
				// send message to show text views
				handler.sendMessage(InGameDialogFrameMessage.getSetAllTextViewsVisibleMessage());
			}
		};
		Thread t = new Thread(runnable);
		t.start();
	}
	
	
	/**
	 * This method flash text in the dialog frame and send a list of messages to DialogFrame's thread 
	 * at the end of the flashing.
	 * 
	 * It is based on a separate thread that send message to the DialogFrame's one,
	 * to avoid freezing while calling Thread.sleep(FLASHING_DURATION) method.
	 *  
	 * @param text the text to flash.
	 */
	
	public void flashOtherCharacterTextAndCompute(CharSequence text, final List<Message> messagesList){
		// used to parse the text
		final StringTokenizer tokenizer = new StringTokenizer((String)text);
		// special thread to flash the text
		Runnable runnable = new Runnable() {
			public void run() {
				flashTokens(tokenizer);
				for (Message message : messagesList) {
					// send message to handler
					handler.sendMessage(message);
				}
			}
		};
		Thread t = new Thread(runnable);
		t.start();
	}
	
	
	private void flashTokens(StringTokenizer tokenizer){
		while(tokenizer.hasMoreTokens()){
			// send a message to the thread that create the text view in order to append text
			handler.sendMessage(InGameDialogFrameMessage.sendOtherCharacterTextFlashMessage(tokenizer.nextToken()));
			// sleep for a while
			try {
				Thread.sleep(FLASHING_DURATION);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
}
