package org.playonline.ui.basic.dialog;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.playonline.ui.basic.R;
import org.playonline.ui.basic.dialog.handler.InGameDialogFrameHandler;
import org.playonline.ui.basic.dialog.handler.InGameDialogFrameMessage;
import org.playonline.ui.basic.map.PlayOnlineMapActivity;
import org.playonline.ui.basic.map.handler.MapViewMessage;
import org.playonline.ui.basic.profile.InventoryActivity;

import android.app.Dialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.graphics.Typeface;
import android.os.Message;
import android.util.Log;
import android.util.TypedValue;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnLongClickListener;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ScrollView;
import android.widget.TextView;

public class InGameDialogFrame implements OnClickListener, OnLongClickListener{

	public 	static final String	CURRENT_DIALOG_FRAME_DATA_ID_KEY 	= "currentDialogFrameDataId";
	public 	static final int	NO_TEXT_VIEW_TO_HIGHLIGHT 			= -1;
	private static final float 	TEXT_SIZE_INCREASEMENT_COEF 		= 1.00f;
	private static float 		DEFAULT_PIXEL_TEXT_SIZE;// initialize on the first instantiation of this class

	private final Dialog dialog;
	private final Context context;
	private final InGameDialogFrameHandler handler;
	private final FlashingTextView flashingTextView;
	private final ImageView otherCharacterImage;
	private final ImageView myCharacterImage;
	private final TextView  otherCharacterText;
	private final ScrollView otherCharacterScrollView;
	private final TextView  firstOptionText;
	private final TextView  secondOptionText;
	private final TextView  thirdOptionText;
	private final TextView  fourthOptionText;
	private final List<TextView> textViewsList;
	private final Map<Integer,Short> followingDataIdByTextViewId;
	private short currentDialogFrameDataId;


	private InGameDialogFrame(Context context) {
		dialog = new Dialog(context);
		this.context = context;
		// for HTC Desire HD and Nexus (One and S)
		dialog.setContentView(R.layout.custom_dialog);
		// for Galaxy Tab
		//dialog.setContentView(R.layout.custom_dialog_galaxy_tab_8_9);
		textViewsList = new Vector<TextView>();
		followingDataIdByTextViewId = new HashMap<Integer, Short>();
		// retrieval of views
		otherCharacterImage = (ImageView) 	dialog.findViewById(R.id.image_other_character);
		myCharacterImage 	= (ImageView)	dialog.findViewById(R.id.image_my_character);
		otherCharacterText 	= (TextView) 	dialog.findViewById(R.id.text_other_character);
		firstOptionText		= (TextView) 	dialog.findViewById(R.id.firstOptionText);
		secondOptionText	= (TextView) 	dialog.findViewById(R.id.secondOptionText);
		thirdOptionText		= (TextView) 	dialog.findViewById(R.id.thirdOptionText);
		fourthOptionText	= (TextView) 	dialog.findViewById(R.id.fourthOptionText);
		// add text views to list
		textViewsList.add(firstOptionText);
		textViewsList.add(secondOptionText);
		textViewsList.add(thirdOptionText);
		textViewsList.add(fourthOptionText);
		// set text views OnClickListener
		firstOptionText.setOnClickListener(this);
		secondOptionText.setOnClickListener(this);
		thirdOptionText.setOnClickListener(this);
		fourthOptionText.setOnClickListener(this);
		// set text views OnLongClickListener
		firstOptionText.setOnLongClickListener(this);
		secondOptionText.setOnLongClickListener(this);
		thirdOptionText.setOnLongClickListener(this);
		fourthOptionText.setOnLongClickListener(this);
		// retrieve default text size
		initDefaultTextSizeValue();
		// instantiation of buttons
		initInventoryButton();
		initGTalkButton();
		initBackButton();
		// and of DialogFrameHandler
		handler = InGameDialogFrameHandler.getInstance(this);
		// instantiation of FlashingTextView
		flashingTextView = FlashingTextView.getInstance(handler);
		// retrieve scroll view
		otherCharacterScrollView = (ScrollView)dialog.findViewById(R.id.scroll_view_other_character);
	}
	

	public static InGameDialogFrame getInstance(Context context){
		return new InGameDialogFrame(context);
	}
	
	
	public static InGameDialogFrame getInstance(Context context, short dialogFrameDataId, boolean textViewsDisabled){
		InGameDialogFrame dialogFrame = new InGameDialogFrame(context);
		dialogFrame.setData(InGameDialogFrameData.getDialogFrameDataByID(dialogFrameDataId));
		if(textViewsDisabled){
			dialogFrame.disableAllTextView();
		}
		return dialogFrame;
	}


	public Dialog getDialog(){
		return dialog;
	}

	
	public void onClick(View v) {
		// set all option text view to default style
		setAllOptionTextViewsToDefaultStyle();
		for (TextView  textView : textViewsList) {
			if(v.getId() == textView.getId()){
				// increase text size
				((TextView) v).setTextSize(TypedValue.COMPLEX_UNIT_PX, DEFAULT_PIXEL_TEXT_SIZE * TEXT_SIZE_INCREASEMENT_COEF);
				// setting type face to bold
				((TextView) v).setTypeface(Typeface.DEFAULT_BOLD);
			}
		}
	}


	public boolean onLongClick(View v) {
		// update dialog frame
		boolean shallOptionTextViewStayEnabled = updateData((TextView)v);
		// if the option text view is intended to stay visible during the other character speech
		if(!shallOptionTextViewStayEnabled){
			//show the text view
			v.setVisibility(View.VISIBLE);
		}
		return false;
	}
	
	
	public TextView getOtherCharacterText() {
		return otherCharacterText;
	}


	public ScrollView getOtherCharacterScrollView() {
		return otherCharacterScrollView;
	}


	public List<TextView> getTextViewsList() {
		return textViewsList;
	}
	
	
	public PlayOnlineMapActivity getPlayOnlineMapActivity(){
		return (PlayOnlineMapActivity)context;
	}

	private void disableAllTextView(){
		for(TextView textView : textViewsList){
			textView.setEnabled(false);
		}
	}
	
	public void setAllTextViewVisibility(int visibility){
		for(TextView textView : textViewsList){
			textView.setVisibility(visibility);
		}
	}
	
	
	public void setAllOptionTextViewsToDefaultStyle(){
		for (TextView textView : textViewsList) {
			// setting text view to its default appearance
			textView.setTextSize(TypedValue.COMPLEX_UNIT_PX,DEFAULT_PIXEL_TEXT_SIZE);
			textView.setTypeface(Typeface.DEFAULT);
		}
	}

	
	public void cancel(){
		dialog.cancel();
		// scroll text to the beginning
		otherCharacterScrollView.scrollTo(0, 0);
	}

	private void setData(InGameDialogFrameData data){
		// current dialog id
		currentDialogFrameDataId = data.getId();
		// title
		dialog.setTitle(data.getTitle());
		// character drawable
		otherCharacterImage.setImageResource(data.getOtherCharacterDrawable());
		// lower character drawable
		myCharacterImage.setImageResource(data.getLowerCharacterDrawable());
		// option text views
		displayAllTextViews(data, false);
		// hide all text views
		setAllTextViewVisibility(View.INVISIBLE);
		// following data
		putAllFollowingData(data);
		// clear other character text
		otherCharacterText.setText("");
		// flash other character text view
		flashingTextView.flashOtherCharacterText(data.getOtherCharacterText());
	}
	
	/*
	 * This method is used when some action have to be performed at the end of the flashing
	 * of the other character text.
	 * 
	 * @param data
	 * @param messagesList 	the list of messages that will be sent in callback to the current thread 
	 * 						when the flashing of the text will be finish by another thread.
	 * 						
	 */
	private boolean computeNewData(InGameDialogFrameData data, List<Message> messagesList, boolean hideDuringSpeech){
		// current dialog id
		currentDialogFrameDataId = data.getId();
		// title
		dialog.setTitle(data.getTitle());
		// character drawable
		otherCharacterImage.setImageResource(data.getOtherCharacterDrawable());
		// lower character drawable
		myCharacterImage.setImageResource(data.getLowerCharacterDrawable());
		// option text views
		displayAllTextViews(data,hideDuringSpeech);
		if(hideDuringSpeech){
			// hide all text views
			setAllTextViewVisibility(View.INVISIBLE);
		}
		// following data
		putAllFollowingData(data);
		// clear other character text
		otherCharacterText.setText("");
		// flash other character text view with a list of message to execute when text is all displayed.
		flashingTextView.flashOtherCharacterTextAndCompute(data.getOtherCharacterText(), messagesList);
		// notice if option text view shall stay enabled
		return data.shallOptionTextViewStayEnabled();
	}
	

	private void initInventoryButton(){
		Button inventoryButton = (Button) dialog.findViewById(R.id.inventoryButton);
		inventoryButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				Intent intent = new Intent();
				intent.setClass(getPlayOnlineMapActivity(), InventoryActivity.class);
				intent.putExtra(CURRENT_DIALOG_FRAME_DATA_ID_KEY, currentDialogFrameDataId);
				getPlayOnlineMapActivity().startActivityForResult(intent,PlayOnlineMapActivity.IVENTORY_REQUEST_CODE);
			}
		});
	}
	
	
	
	
	private void initGTalkButton(){
		final ImageButton button = (ImageButton) dialog.findViewById(R.id.buttonGTalkDialog);
        button.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
            	Intent intent = new Intent("android.intent.action.MAIN");
            	intent.addCategory("android.intent.category.LAUNCHER");
                intent.setComponent(ComponentName.unflattenFromString(PlayOnlineMapActivity.GTALK_COMPONENT_NAME));
                dialog.getContext().startActivity(intent);  
            }
        });
	}
	
	private void initBackButton(){
		ImageButton backButton = (ImageButton) dialog.findViewById(R.id.backButton);
		backButton.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				// if character is attached to leader, it cannot cancel the dialog
				if(((PlayOnlineMapActivity)context).getCharacter().isAttachedToLeader()){
					String message = "Vous ne pouvez pas quitter le dialogue du personnage auquel vous êtes attaché.";
					((PlayOnlineMapActivity)context).getMapViewHandler().sendMessage(MapViewMessage.displayLongToastMessage(message));
				}else{
					cancel();
					// if character is leader, dialog frame is canceled for its group members
					if(((PlayOnlineMapActivity)context).getCharactersManager().isMyCharacterLeader()){
						((PlayOnlineMapActivity)context).getGroupManager().cancelGroupDialogFrame();
					}
				}
			}
		});
	}


	private void initDefaultTextSizeValue(){
		if(DEFAULT_PIXEL_TEXT_SIZE== 0){
			DEFAULT_PIXEL_TEXT_SIZE = firstOptionText.getTextSize();
		}
	}


	/*
	 * shouldStayInvisible boolean is usefull for group members.
	 * Indeed, for group members option text view are set to View.INVISIBLE, and should not be set back to visible here.
	 */
	private void displayAllTextViews(InGameDialogFrameData data, boolean shouldStayInvisible){
		displayTextViewIfNecessary(firstOptionText, data.getFirstOptionText(),shouldStayInvisible);
		displayTextViewIfNecessary(secondOptionText, data.getSecondOptionText(),shouldStayInvisible);
		displayTextViewIfNecessary(thirdOptionText, data.getThirdOptionText(),shouldStayInvisible);
		displayTextViewIfNecessary(fourthOptionText, data.getFourthOptionText(),shouldStayInvisible);
	}
	
	
	private void putAllFollowingData(InGameDialogFrameData data){
		putFollowingDataIdIfNecessary(firstOptionText.getId(), data.getFirstOptionFollowingDialogId());
		putFollowingDataIdIfNecessary(secondOptionText.getId(), data.getSecondOptionFollowingDialogId());
		putFollowingDataIdIfNecessary(thirdOptionText.getId(), data.getThirdOptionFollowingDialogId());
		putFollowingDataIdIfNecessary(fourthOptionText.getId(), data.getFourthOptionFollowingDialogId());	
	}


	/*
	 * shouldStayInvisible boolean is usefull for group members.
	 * Indeed, for group members option text view are set to View.INVISIBLE, and should not be set back to visible here.
	 */
	private void displayTextViewIfNecessary(TextView textView, CharSequence text, boolean shouldStayInvisible){
		if(text == null){
			textView.setVisibility(View.GONE);
		}else{
			// for group members, textView should not be set to visible
			if(shouldStayInvisible){
				textView.setVisibility(View.VISIBLE);
			}
			textView.setText(text);
		}
	}
	

	private void putFollowingDataIdIfNecessary(int textViewId, short followingDataId){
		if(followingDataId != 0){
			followingDataIdByTextViewId.put(Integer.valueOf(textViewId), Short.valueOf(followingDataId));
		}
	}
	
	
	/*
	 * This method successively:
	 * - prepare messages for each new elements if there are some
	 * - update the dialog's data
	 * - if character is leader, new dialog is sent to group members
	 * - ask to compute the message at the end of the other character flashing text.
	 */
	private boolean updateData(TextView optionTextViewSelected){
		// TODO NullPointerException here...
		// retrieve the next dialog data id matching with text view
		if(followingDataIdByTextViewId != null){
			Short dialogFrameDataId = followingDataIdByTextViewId.get(Integer.valueOf(optionTextViewSelected.getId()));
			// if following data Id field has been initialized
			if(dialogFrameDataId != null){
				List<Message> messagesList = unblockNewElements(dialogFrameDataId);
				// two messages needed for every dialog
				messagesList.add(InGameDialogFrameMessage.getSetAllTextViewsVisibleMessage());
				messagesList.add(InGameDialogFrameMessage.getSetAllOptionsTextViewToDefaultStyleMessage());
				// retrieval of following DialogframeData
				InGameDialogFrameData followingData = InGameDialogFrameData.getDialogFrameDataByID(dialogFrameDataId.shortValue());
				if(followingData != null){
					if(!followingData.shallOptionTextViewStayEnabled()){
						// following message is inserted only if option text shall be disabled
						messagesList.add(InGameDialogFrameMessage.getDisableViewMessage(optionTextViewSelected.getId()));
					}
					// if my character is leader, new dialog is sent to group members
					if(((PlayOnlineMapActivity)context).getCharactersManager().isMyCharacterLeader()){
						int textViewIndex = getTextViewIndex(optionTextViewSelected);
						((PlayOnlineMapActivity)context).getGroupManager().updateGroupDialogFrame(textViewIndex, dialogFrameDataId.shortValue());
					}
					// update dialog data with the given message list
					return computeNewData(followingData, messagesList, true);
				}else{
					// there is no following dialog frame
					return true;
				}
			}// if dialog frame ID is invalid
			return false;
		}// if dialogFrameDataId is null
		return false;
	}
	
	private int getTextViewIndex(TextView optionTextViewSelected){
		int index = -1;
		for (int i = 0; i < textViewsList.size(); i++) {
			if(textViewsList.get(i).equals(optionTextViewSelected)){
				index = i;
				break;
			}
		}
		Log.i(getClass().getSimpleName(), "TEXT VIEW INDEX = "+index);
		return index;
	}
	
	/**
	 * Should be used for group members.
	 * 
	 * @param dialogFrameDataId
	 * @return
	 */
	public boolean updateData(int textViewIndex, short dialogFrameDataId){
		// this test is done to avoid a recursion if leader or follower attempts to update dialog with RMAI.
		if(dialogFrameDataId != currentDialogFrameDataId){
			List<Message> messagesList = unblockNewElements(dialogFrameDataId);
			// two messages needed for every dialog
			messagesList.add(InGameDialogFrameMessage.getSetAllTextViewsVisibleMessage());
			messagesList.add(InGameDialogFrameMessage.getSetAllOptionsTextViewToDefaultStyleMessage());
			// retrieval of following DialogframeData
			InGameDialogFrameData followingData = InGameDialogFrameData.getDialogFrameDataByID(dialogFrameDataId);
			highlightTextView(textViewIndex);
			// if my character is leader, new dialog is sent to group members
			if(((PlayOnlineMapActivity)context).getCharactersManager().isMyCharacterLeader()){
				((PlayOnlineMapActivity)context).getGroupManager().updateGroupDialogFrame(NO_TEXT_VIEW_TO_HIGHLIGHT, dialogFrameDataId);
			}
			else if(((PlayOnlineMapActivity)context).getCharacter().isAttachedToLeader()){
				((PlayOnlineMapActivity)context).getGroupManager().followerUpdateGroupDialogFrame(NO_TEXT_VIEW_TO_HIGHLIGHT, dialogFrameDataId);
			}
			// update dialog data with the given message list
			return computeNewData(followingData, messagesList, false);
		}else{
			return false;
		}
		
	}
	
	
	private void highlightTextView(int textViewIndex){
		// set all option text view to default style
		setAllOptionTextViewsToDefaultStyle();
		setAllTextViewVisibility(View.INVISIBLE);
		if(textViewIndex != NO_TEXT_VIEW_TO_HIGHLIGHT){
			// increase text size
			textViewsList.get(textViewIndex).setTextSize(TypedValue.COMPLEX_UNIT_PX, DEFAULT_PIXEL_TEXT_SIZE * TEXT_SIZE_INCREASEMENT_COEF);
			// setting type face to bold
			textViewsList.get(textViewIndex).setTypeface(Typeface.DEFAULT_BOLD);
			// setting the required text view visibility
			textViewsList.get(textViewIndex).setVisibility(View.VISIBLE);
		}
	}
	
	
	/**
	 * New elements (places, areas and items) are unblocked here, depending on the id of the dialog.
	 * 
	 * @param dialogFrameDataId
	 * @return
	 */
	private List<Message> unblockNewElements(short dialogFrameDataId){
		List<Message> messagesList = new Vector<Message>();
		switch (dialogFrameDataId) {
		case 2:
			// unblock plantation
			messagesList.add(InGameDialogFrameMessage.getDisplayPlantationMessage());
			break;
		case 5:
			// display new element in inventory
			messagesList.add(InGameDialogFrameMessage.getNewElementInInventoryMessage());
			break;
		/*case 7:
			// display new element in inventory
			messagesList.add(InGameDialogFrameMessage.getUnknownAreaMessage());
			break;*/
		
		default:
			break;
		}
		return messagesList;
	}
}
