package org.playonline.ui.basic.dialog;

import java.util.List;
import java.util.StringTokenizer;

import org.playonline.ui.basic.map.PlayOnlineMapActivity;
import org.playonline.ui.basic.profile.Character;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.text.method.DialerKeyListener;
import android.widget.EditText;

import com.google.android.maps.GeoPoint;

public class MapViewDialogsManager {

	private PlayOnlineMapActivity mapActivity;
	private final AlertDialog.Builder messageBuilder;
	private final AlertDialog.Builder joinGroupBuilder;

	
	private MapViewDialogsManager(PlayOnlineMapActivity mapActivity){
		this.mapActivity = mapActivity;
		messageBuilder = getMessageBuilder();
		joinGroupBuilder = getJoinGroupBuilder();
	}

	
	public static MapViewDialogsManager getInstance(PlayOnlineMapActivity mapActivity){
		return new MapViewDialogsManager(mapActivity);
	}

	
	public void displayTravelDialogToPlayOnlineLocation(String title, final String snippet, 
														final int byWalkTime, final int byWalkPoints,
														final int byBikeTime, final int byBikePoints,
														final int byCarTime, final int byCarPoints){
		 // GALAXY TAB and HTC Desire HD
        /*final CharSequence[] items ={   "A pied : "+"              "+byWalkTime+" mn    -    "+byWalkPoints+" points",
                                                                        "En velo : "+"              "+byBikeTime+" mn    -    "+byBikePoints+" points",
                                                                        "En voiture : "+"          "+byCarTime+" mn    -    "+byCarPoints+" points"};*/
        
        // Nexus
       final CharSequence[] items ={ "A pied : "+"     "+byWalkTime+" mn -  "+byWalkPoints+" pts",
                        "En velo : "+"     "+byBikeTime+" mn - "+byBikePoints+" pts",
                        "En voiture : "+""+byCarTime+" mn - "+byCarPoints+" pts"};
		AlertDialog.Builder builder = new AlertDialog.Builder(mapActivity);
		builder.setTitle(title+" -- "+snippet);
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, final int item) {
				boolean transportEnabled = computeTransportPoints(item, byBikePoints, byCarPoints);
				if(transportEnabled){
					mapActivity.getMapAnimation().travelToPlayOnlineLocation(items[item], snippet);
				}
			}
		});
		AlertDialog alert = builder.create();
		alert.show();
	}
	
	
	public void displayUGASPAddressDialog(){
		final AlertDialog.Builder alert = new AlertDialog.Builder(mapActivity);
		final EditText input = new EditText(mapActivity);
		input.setKeyListener(new DialerKeyListener());
		alert.setView(input);
		alert.setTitle("uGASP address");
		alert.setPositiveButton("Ok", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int whichButton) {
				String value = input.getText().toString().trim();
				mapActivity.getDBAdapter().putAddress(value);
			}
		});
		alert.show();
	}
	
	
	private boolean computeTransportPoints(int item,int byBikePoints,int byCarPoints){
		boolean transportEnabled = true;
		// bike selected
		if (item == 1){
			transportEnabled = computeCarTransport(byBikePoints);
		}
		// car selected
		else if (item == 2){
			transportEnabled = computeHelicoTransport(byCarPoints);
		}
		return transportEnabled;
	}


	private boolean computeCarTransport(int byBikePoints) {
		boolean bikeEnabled = true;
		short availablePoints = mapActivity.getCharacter().getTransportPoints();
		if(byBikePoints>availablePoints){
			bikeEnabled = false;
			displayAlertMessage("Il vous manque "+(byBikePoints-availablePoints)+" points pour utiliser la voiture.");
		}else{
			mapActivity.setTransportPoints((short)(availablePoints - byBikePoints));
		}
		return bikeEnabled;
	}

	
	private boolean computeHelicoTransport(int byCarPoints) {
		boolean carEnabled = true;
		short availablePoints = mapActivity.getCharacter().getTransportPoints();
		if(byCarPoints>availablePoints){
			carEnabled = false;
			displayAlertMessage("Il vous manque "+(byCarPoints-availablePoints)+" points pour utiliser la voiture.");
		}else{
			mapActivity.setTransportPoints((short)(availablePoints - byCarPoints));
		}
		return carEnabled;
	}
	

	public void displayAlertMessage(final String message){
		messageBuilder.setMessage(message);
		// display dialog
		final AlertDialog alert = messageBuilder.create();
		alert.show();
	}


	private void displayJoinGroupMessage(final Short playerActorSessionId, final String playerName){
		// customize builder
		joinGroupBuilder.setTitle("Se joindre à "+playerName+" ?");
		joinGroupBuilder.setPositiveButton("Oui", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int id) {
				// join user
				mapActivity.getGroupManager().joinCharacter(playerActorSessionId);
			}
		});
		// display dialog
		final AlertDialog alert = joinGroupBuilder.create();
		alert.show();
	}
	
	
	public void displayLeaveGroupMessage(){
		String leaderName = mapActivity.getCharactersManager().getNameByActorSessionId(mapActivity.getCharacter().getLeaderActorSessionID());
		joinGroupBuilder.setTitle("Quitter "+leaderName+ " ?");
		joinGroupBuilder.setPositiveButton("Oui", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int id) {
				// leave leader
				mapActivity.getGroupManager().leaveLeader();
			}
		});
		// display dialog
		final AlertDialog alert = joinGroupBuilder.create();
		alert.show();
	}
	

	public void displayTravelDialogToStandardLocation(final GeoPoint destination){
		final CharSequence[] items =  {"A pied", "En velo", "En voiture"};;
		AlertDialog.Builder builder = new AlertDialog.Builder(mapActivity);
		builder.setTitle("Choisir un moyen de transport");
		builder.setItems(items, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, final int item) {
				mapActivity.getMapAnimation().travelToStandardLocation(destination);
			}
		});
		AlertDialog alert = builder.create();
		alert.show();
	}
	
	
	public void displayCharactersInMyLocation(){
		// suitable data are suffixed to character names
		final CharSequence[] charactersArray = buildProperCharacterItems();
		AlertDialog.Builder builder = new AlertDialog.Builder(mapActivity);
		builder.setTitle("Personnages présents");
		builder.setItems(charactersArray, new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, final int item) {
				String characterItem = (String)charactersArray[item];
				computeSuitableInteraction(characterItem);
			}
		});
		AlertDialog alert = builder.create();
		alert.show();
	}
	
	
	private final CharSequence[] buildProperCharacterItems(){
		// retrieval of actorSessionIds of characters located on my location
		List<Short> actorSessionIdsList = mapActivity.getCharactersManager().getCharactersListInLocation(mapActivity.getCharactersManager().getMyCharacterLocation());
		// collection to store matching characters' informations
		final CharSequence[] characters = new CharSequence[actorSessionIdsList.size()];
		for (int i = 0; i < actorSessionIdsList.size(); i++) {
			Character character = mapActivity.getCharactersManager().getCharacterByActorSessionId(actorSessionIdsList.get(i));
			String name = character.getName();
			// if this character is my leader
			if(mapActivity.getCharacter().isAttachedToLeader() && mapActivity.getCharacter().getLeaderActorSessionID() == actorSessionIdsList.get(i).shortValue()){
				name = GroupTokenItem.getMyLeaderItem(name);
			}// this character is attached to another one
			else if(character.isAttachedToLeader()){
				// if my character is its leader
				if(character.getLeaderActorSessionID() == mapActivity.getUGASPClient().getID().getActorSessionID()){
					name = GroupTokenItem.getMyMemberItem(name);
				}else{
					String leaderName = mapActivity.getCharactersManager().getNameByActorSessionId(character.getLeaderActorSessionID());
					name = GroupTokenItem.getOtherMemberItem(name, leaderName);
				}
			}
			characters[i] = name;
		}
		return characters;
	}
	
	private void computeSuitableInteraction(String characterItem){
		Short characterActorSessionId;
		// if character on item is neither our leader nor a member of my group nor a member of an external group
		if(!GroupTokenItem.isMyLeaderItem(characterItem) && !GroupTokenItem.isMyMemberItem(characterItem)  && !GroupTokenItem.isOtherMemberItem(characterItem)){
			// retrieval of proper name
			String name = GroupTokenItem.getCharacterNameWithoutGroupToken(characterItem);
			// retrieval of its actorSessionId
			characterActorSessionId = mapActivity.getCharactersManager().getActorSessionIdByName(name);
			if(characterActorSessionId != null){
				displayJoinGroupMessage(characterActorSessionId, name);
			}
		}// if character is my leader
		else if(GroupTokenItem.isMyLeaderItem(characterItem)){
			displayLeaveGroupMessage();
		}
	}
	

	private AlertDialog.Builder getMessageBuilder(){
		// build dialog box to display when user clicks the flag
		AlertDialog.Builder builder = new AlertDialog.Builder(mapActivity);
		builder.setCancelable(false);
		builder.setIcon(android.R.drawable.ic_dialog_alert);
		builder.setTitle("Alert");
		builder.setPositiveButton("OK", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int id) {
				dialog.dismiss();
			}
		});
		return builder;
	}


	private AlertDialog.Builder getJoinGroupBuilder(){
		AlertDialog.Builder builder = new AlertDialog.Builder(mapActivity);
		builder.setCancelable(false)
		.setNegativeButton("No", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialog, int id) {
				dialog.cancel();
			}
		});
		return builder;
	}
	
	
	
	
	private static class GroupTokenItem{
		
		private final static String	DELIM 				= "\n";
		private final static String MY_LEADER_TOKEN		= DELIM+" vous suivez ce personnage";
		private final static String MY_MEMBER_TOKEN		= DELIM+" suit votre personnage";
		private final static String OTHER_MEMBER_TOKEN 	= DELIM+" suit le personnage ";
		
		
		
		static String getMyLeaderItem(String leaderName){
			return leaderName+MY_LEADER_TOKEN;
		}
		
		
		static String getMyMemberItem(String memberName){
			return memberName+MY_MEMBER_TOKEN;
		}
		
		
		static String getOtherMemberItem(String memberName, String leaderName){
			return memberName+OTHER_MEMBER_TOKEN+leaderName;
		}
		
		static boolean isMyLeaderItem(String characterName){
			boolean isGroupLeader = false;
			if(characterName.contains(MY_LEADER_TOKEN)){
				isGroupLeader = true;
			}
			return isGroupLeader;
		}
		
		
		static boolean isMyMemberItem(String characterName){
			boolean isMyMember = false;
			if(characterName.contains(MY_MEMBER_TOKEN)){
				isMyMember = true;
			}
			return isMyMember;
		}
		
		
		static boolean isOtherMemberItem(String characterName){
			boolean isGroupMember = false;
			if(characterName.contains(OTHER_MEMBER_TOKEN)){
				isGroupMember = true;
			}
			return isGroupMember;
		}

		
		static String getCharacterNameWithoutGroupToken(String characterNameWithToken){
			String characterNameWithoutToken = null;
			if(isMyLeaderItem(characterNameWithToken) || isOtherMemberItem(characterNameWithToken)){
				StringTokenizer tokenizer = new StringTokenizer(characterNameWithToken);
				// first token matches with the user name
				characterNameWithoutToken = tokenizer.nextToken();
			}else{
				characterNameWithoutToken = characterNameWithToken;
			}
			return characterNameWithoutToken;
		}
	
		
		/*static String getGroupLeaderFromGroupMemberItem(String groupMemberItem){
		String groupLeader = null;
		if(isOtherMemberItem(groupMemberItem)){
			StringTokenizer tokenizer = new StringTokenizer(groupMemberItem);
			// leader matches with the last token
			while(tokenizer.hasMoreTokens()){
				groupLeader = tokenizer.nextToken();
			}
		}
		return groupLeader;
		}*/
	
	}

}
