package org.playonline.ui.basic.dialog;

import org.playonline.ui.basic.R;
import org.playonline.ui.basic.map.PlayOnlineMapActivity;
import org.playonline.ui.basic.map.handler.MapViewMessage;
import org.playonline.ui.basic.profile.InventoryActivity;

import com.google.android.maps.GeoPoint;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Message;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

public class NewElementsAlertDialog {

	private PlayOnlineMapActivity mapActivity;

	private NewElementsAlertDialog(PlayOnlineMapActivity mapActivity){
		this.mapActivity = mapActivity;
	}

	public static NewElementsAlertDialog getInstance(PlayOnlineMapActivity mapActivity){
		return new NewElementsAlertDialog(mapActivity);
	}

	/**
	 * Creates an alert dialog to inform player about a new place unblocked.
	 * However, if the character is grouped to a leader, only a toast message is displayed in its screen,
	 * in order to avoid a gap between action on the screen of the leader and on the screen of other characters from its group.
	 * @param message
	 * @param newLocation
	 * @param dialogFrame
	 */
	public void displayNewPlace(String message, final GeoPoint newLocation, final Dialog dialogFrame){
		// if character is not attached to a leader
		if(!mapActivity.getCharacter().isAttachedToLeader()){
			displayNewPlaceAlertDialog(message, newLocation, dialogFrame);
		}// otherwise
		else{
			 displayNewPlaceToast(message, newLocation, dialogFrame);
		}
	}
	
	private void displayNewPlaceAlertDialog(String message, final GeoPoint newLocation, final Dialog dialogFrame){
		AlertDialog.Builder builder = new AlertDialog.Builder(mapActivity);
		builder.setMessage(message)
		.setCancelable(false)
		.setPositiveButton("Go", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialogAlert, int id) {
				mapActivity.getMapAnimation().showNewLocation(newLocation,dialogFrame);
			}
		});
		AlertDialog alert = builder.create();
		alert.show();
	}
	
	
	private void displayNewPlaceToast(String message, final GeoPoint newLocation, final Dialog dialogFrame){
		Message msg = MapViewMessage.displayLongToastMessage(message);
		mapActivity.getMapViewHandler().sendMessage(msg);mapActivity.getMapAnimation().showNewLocation(newLocation,dialogFrame);
	}

	/**
	 * Creates an alert dialog to inform player about a new element unblocked, available in the inventory.
	 * However, if the character is grouped to a leader, only a toast message is displayed in its screen,
	 * in order to avoid a gap between action on the screen of the leader and on the screen of other characters from its group.
	 * @param message
	 * @param newLocation
	 * @param dialogFrame
	 */
	
	public void displayNewElementInInventory(final String title, final String description, final int drawableId){
		// if character is not attached to a leader
		if(!mapActivity.getCharacter().isAttachedToLeader()){
			displayNewElementInInventoryAlertDialog(title, description, drawableId);
		}// otherwise
		else{
			displayNewElementInInventoryToast(title, description, drawableId);
		}
	}
	
	private void displayNewElementInInventoryAlertDialog(final String name, final String description, final int drawableId){
		AlertDialog.Builder builder = new AlertDialog.Builder(mapActivity);

		LayoutInflater inflater = (LayoutInflater) mapActivity.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		// bind the layout view with the new_item_layout
		View layout = inflater.inflate(	R.layout.new_item_layout, null);

		ImageView image = (ImageView) layout.findViewById(R.id.newItemImage);
		image.setImageResource(drawableId);
		
		TextView text = (TextView) layout.findViewById(R.id.newItemText);
		text.setText(name+" débloqué !");

		builder.setView(layout)
		.setCancelable(false)
		.setPositiveButton("Go", new DialogInterface.OnClickListener() {
			public void onClick(DialogInterface dialogAlert, int id) {
				InventoryActivity.addElement(name, description, drawableId);
			}
		});

		AlertDialog alertDialog = builder.create();
		alertDialog.show();
	}
	
	
	private void displayNewElementInInventoryToast(final String name, final String description, final int drawableId){
		Message msg = MapViewMessage.displayLongToastMessage(name+" débloqué !");
		mapActivity.getMapViewHandler().sendMessage(msg);
		InventoryActivity.addElement(name, description, drawableId);
	}
}
