package org.playonline.ui.basic.dialog.handler;

import org.playonline.ui.basic.R;
import org.playonline.ui.basic.dialog.InGameDialogFrame;
import org.playonline.ui.basic.interaction.InGameProgress;
import org.playonline.ui.basic.map.PlayOnlineGeoPoint;

import com.google.android.maps.GeoPoint;

import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.widget.TextView;

public class InGameDialogFrameHandler extends Handler{
	
	private final InGameDialogFrame dialogFrame;
	
	private InGameDialogFrameHandler(InGameDialogFrame dialogFrame){
		this.dialogFrame = dialogFrame;
	}
	
	public static InGameDialogFrameHandler getInstance(InGameDialogFrame dialogFrame){
		return new InGameDialogFrameHandler(dialogFrame);
	}
	
	
	@Override
	public void handleMessage(Message msg) {
		switch (msg.what) {
		case InGameDialogFrameWhat.OTHER_CHARACTER_TEXT_FLASH_WHAT:
			String text = (String)(msg.getData().getCharSequence(InGameDialogFrameMessage.DEFAULT_BUNDLE_KEY));
			String newText = dialogFrame.getOtherCharacterText().getText()+text+" ";
			// flash text to other text character
			dialogFrame.getOtherCharacterText().setText(newText);
			// scroll text to the bottom
			dialogFrame.getOtherCharacterScrollView().smoothScrollTo(0, dialogFrame.getOtherCharacterText().getBottom());
			break;
			
		case InGameDialogFrameWhat.SET_ALL_TEXT_VIEWS_VISIBLE_WHAT:
			dialogFrame.setAllTextViewVisibility(View.VISIBLE);
			break;
			
		case InGameDialogFrameWhat.SET_ALL_OPTIONS_TEXT_VIEWS_TO_DEFAULT_STYLE_WHAT:
			dialogFrame.setAllOptionTextViewsToDefaultStyle();
			break;
			
		case InGameDialogFrameWhat.DISABLE_VIEW_WHAT:
			int viewID = msg.getData().getInt(InGameDialogFrameMessage.DEFAULT_BUNDLE_KEY);
			for (TextView textView: dialogFrame.getTextViewsList()) {
				if(textView.getId() == viewID){
					textView.setEnabled(false);
					break;
				}
			}
			break;
			
		case InGameDialogFrameWhat.DISPLAY_FOREST_WHAT:
			//unblockNewPlace(R.drawable.foret_tetra, PlayOnlineGeoPoint.SAINT_BENOIT_PLANTATION,"Ile de la Réunion", "Forêt",InGameProgress.fieldsWorkerFirstDialog);
			unblockNewPlace(R.drawable.foret_tetra, PlayOnlineGeoPoint.PLANTATION_AREA,"Ile Graham", "Forêt",InGameProgress.fieldsWorkerFirstDialog);
			
			break;
		
		case InGameDialogFrameWhat.DISPLAY_PLANTATION_WHAT:
			unblockNewPlace(R.drawable.plantation_tetra, PlayOnlineGeoPoint.PLANTATION_AREA,"Ile de la Réunion", "Plantations",InGameProgress.fieldsWorkerFirstDialog);
			break;
			
		case InGameDialogFrameWhat.DISPLAY_PASS_IN_INVENTORY_WHAT:
			unblockNewElement("Autorisation spéciale", 
								"Carte confiée par le maire de l'île.\n"+
								"Peut s'avérer utile pour avoir la confiance des habitants.", 
								R.drawable.carte_magnetique);
			break;
			
		case InGameDialogFrameWhat.DISPLAY_HIDDEN_AREA_WHAT:
			unblockNewArea(PlayOnlineGeoPoint.SOUTH_ISLAND_AREA, "Sud de l'île");
			break;
			
		default:
			break;
		}
	}
	
	
	private void unblockNewPlace(int drawableId, GeoPoint location, String title, String snippet, short matchingDialogId){
		// create a new place
		dialogFrame.getPlayOnlineMapActivity().unblockNewPlace(drawableId, location, title, snippet,matchingDialogId);
		// show this new place
		dialogFrame.getPlayOnlineMapActivity().getNewElementsAlertDialog().displayNewPlace("Nouveau lieu débloqué : "+snippet+" !", location, dialogFrame.getDialog());
	}
	
	private void unblockNewArea(GeoPoint location, String title){
		// create a new place
		dialogFrame.getPlayOnlineMapActivity().unblockNewArea(location, title);
		// show this new place
		dialogFrame.getPlayOnlineMapActivity().getNewElementsAlertDialog().displayNewPlace("Nouvelle zone à parcourir: "+title, location, dialogFrame.getDialog());
	}
	
	
	private void unblockNewElement(String name, String description, int marker){
		// add and display new element
		dialogFrame.getPlayOnlineMapActivity().getNewElementsAlertDialog().displayNewElementInInventory(name, description, marker);
	}
	
}
