package org.playonline.ui.basic.interaction;

import java.util.List;

import org.playonline.network.client.RMAIClientManager;
import org.playonline.ui.basic.map.PlayOnlineMapActivity;
import org.playonline.ui.basic.profile.Character;

public class GroupManager {
	
	private PlayOnlineMapActivity mapActivity;
	
	
	private GroupManager(PlayOnlineMapActivity mapActivity) {
		this.mapActivity = mapActivity;
	}
	
	
	public static GroupManager getInstance(PlayOnlineMapActivity mapActivity){
		return new GroupManager(mapActivity);
	}
	
	
	public void joinCharacter(Short leaderActorSessionId) {
		// inform leader of attachment
		if(mapActivity.getUGASPClient() != null){
			mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
					RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(), 
					"characterJoinLeader",
					leaderActorSessionId
					);
			// attach character to leader
			mapActivity.getCharacter().attachToLeader(leaderActorSessionId.shortValue());
		}
	}

	
	public void leaveLeader() {
		// inform leader of detachment
		if(mapActivity.getUGASPClient() != null){
			mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
					RMAIClientManager.ActorSessionID.uGASP_GAME_LOGIC.getID(), 
					"characterLeaveLeader");
			// detach character from leader
			mapActivity.getCharacter().detachFromLeader();
		}
	}
	

	public void showDialogToGroup(short matchingDialogId) {
		// for each character on my location
		List<Short> charactersOnLocationList = mapActivity.getCharactersManager().getCharactersListInLocation(mapActivity.getCharactersManager().getMyCharacterLocation());
		if(charactersOnLocationList != null){
			for (Short actorSessionId : charactersOnLocationList) {
				Character character = mapActivity.getCharactersManager().getCharacterByActorSessionId(actorSessionId);
				// if my character is the leader of this character
				if(character.isAttachedToLeader() && character.getLeaderActorSessionID() == mapActivity.getUGASPClient().getID().getActorSessionID()){
					mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
							actorSessionId.shortValue(), 
							"showDialog",
							Short.valueOf(matchingDialogId));
				}
			}
		}
	}

	public void updateGroupDialogFrame(int textViewIndex, short matchingDialogId) {
		// for each character on my location
		List<Short> charactersOnLocationList = mapActivity.getCharactersManager().getCharactersListInLocation(mapActivity.getCharactersManager().getMyCharacterLocation());
		if(charactersOnLocationList != null){
			for (Short actorSessionId : charactersOnLocationList) {
				Character character = mapActivity.getCharactersManager().getCharacterByActorSessionId(actorSessionId);
				// if my character is the leader of this character
				if(character.isAttachedToLeader() && character.getLeaderActorSessionID() == mapActivity.getUGASPClient().getID().getActorSessionID()){
					mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
							actorSessionId.shortValue(), 
							"updateDialogFrame",
							Integer.valueOf(textViewIndex),
							Short.valueOf(matchingDialogId));
				}
			}
		}
	}
	
	/**
	 * Method used by followers to update a DialogFrame for their group (leader + member).
	 * It is only called when the follower use an element in the inventory (anyway, follower cannot do anything else).
	 * 
	 * @param textViewIndex
	 * @param matchingDialogId
	 */
	public void followerUpdateGroupDialogFrame(int textViewIndex, short matchingDialogId) {
		// for each character on my location
		List<Short> charactersOnLocationList = mapActivity.getCharactersManager().getCharactersListInLocation(mapActivity.getCharactersManager().getMyCharacterLocation());
		if(charactersOnLocationList != null){
			for (Short actorSessionId : charactersOnLocationList) {
				// if character is my leader
				if(actorSessionId.shortValue() == (mapActivity.getCharacter().getLeaderActorSessionID())){
					mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
							actorSessionId.shortValue(), 
							"updateDialogFrame",
							Integer.valueOf(textViewIndex),
							Short.valueOf(matchingDialogId));
				}else{
					Character character = mapActivity.getCharactersManager().getCharacterByActorSessionId(actorSessionId);
					// if character has the same leader than my character
					if(character.isAttachedToLeader() && character.getLeaderActorSessionID() == (mapActivity.getCharacter().getLeaderActorSessionID())){
						mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
								actorSessionId.shortValue(), 
								"updateDialogFrame",
								Integer.valueOf(textViewIndex),
								Short.valueOf(matchingDialogId));
					}
				}
			}
		}
	}


	public void cancelGroupDialogFrame(){
		// for each character on my location
		List<Short> charactersOnLocationList = mapActivity.getCharactersManager().getCharactersListInLocation(mapActivity.getCharactersManager().getMyCharacterLocation());
		if(charactersOnLocationList != null){
			for (Short actorSessionId : charactersOnLocationList) {
				Character character = mapActivity.getCharactersManager().getCharacterByActorSessionId(actorSessionId);
				// if my character is the leader of this character
				if(character.isAttachedToLeader() && character.getLeaderActorSessionID() == mapActivity.getUGASPClient().getID().getActorSessionID()){
					mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
							actorSessionId.shortValue(), 
							"cancelDialogFrame");
				}
			}
		}
	}


	public void makeGroupCatchHiddenElement(String name, String description, int marker, int hiddenElementHashCode){
		// for each character on my location
		List<Short> charactersOnLocationList = mapActivity.getCharactersManager().getCharactersListInLocation(mapActivity.getCharactersManager().getMyCharacterLocation());
		if(charactersOnLocationList != null){
			for (Short actorSessionId : charactersOnLocationList) {
				Character character = mapActivity.getCharactersManager().getCharacterByActorSessionId(actorSessionId);
				// if my character is the leader of this character
				if(character.isAttachedToLeader() && character.getLeaderActorSessionID() == mapActivity.getUGASPClient().getID().getActorSessionID()){
					mapActivity.getUGASPClient().getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
							actorSessionId.shortValue(), 
							"catchHiddenElement",
							name,
							description,
							Integer.valueOf(marker),
							Integer.valueOf(hiddenElementHashCode));
				}
			}
		}
	}

}
