package org.playonline.ui.basic.map;

import org.playonline.ui.basic.map.PlayOnlineGeoPoint;
import org.playonline.ui.basic.map.PlayOnlineMapActivity;
import org.playonline.ui.basic.map.overlay.OtherCharacterOverlay;

import com.google.android.maps.GeoPoint;

import android.app.Dialog;
import android.widget.Toast;

public class MapAnimation {
	
	public static final int	CURRENT_LOCATION_DRAWABLE_ALPHA = 100;
	public static final int	FULLY_OPAQUE_APLHA = 255;
	
	private static final long 	TRANSPORT_DURATION = 1000;//ms
	
	private PlayOnlineMapActivity mapActivity;
	
	private MapAnimation(PlayOnlineMapActivity mapActivity){
		this.mapActivity = mapActivity;
	}
	
	public static MapAnimation getInstance(PlayOnlineMapActivity mapActivity){
		return new MapAnimation(mapActivity);
	}
	
	public void travelToPlayOnlineLocation(final CharSequence transport, final String place){
		mapActivity.getOverlayManager().getCharacterOverlay().setAlpha(FULLY_OPAQUE_APLHA);
		// animate to former player location
    	mapActivity.getMapController().animateTo(mapActivity.getOverlayManager().getCharacterOverlay().getLocation(),new Runnable() {	
			public void run() {
		    	try {
					Thread.sleep(TRANSPORT_DURATION);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				// animate to new player location
				mapActivity.getCharactersManager().setMyCharacterLocation(PlayOnlineGeoPoint.getGeoPointFromName(place));
				mapActivity.getMapController().animateTo(mapActivity.getCharactersManager().getMyCharacterLocation(), new Runnable() {
					public void run() {
						// change marker of overlay to group member marker
						mapActivity.getOverlayManager().getCharacterOverlay().setAlpha(CURRENT_LOCATION_DRAWABLE_ALPHA);
						Toast.makeText(mapActivity, "Vous êtes arrivé à "+place, Toast.LENGTH_SHORT).show();
					}
				});     
			}
		});
	}
	
	
	public void travelToStandardLocation(final GeoPoint destination){
		mapActivity.getOverlayManager().getCharacterOverlay().setAlpha(FULLY_OPAQUE_APLHA);
		// animate to former player location
    	mapActivity.getMapController().animateTo(mapActivity.getOverlayManager().getCharacterOverlay().getLocation(),new Runnable() {	
			public void run() {
		    	try {
					Thread.sleep(TRANSPORT_DURATION);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				// animate to new player location
				mapActivity.getCharactersManager().setMyCharacterLocation(destination);
				mapActivity.getMapController().animateTo(mapActivity.getCharactersManager().getMyCharacterLocation());   
			}
		});
	}
	
	
	public void travelToGroupLeader(final OtherCharacterOverlay otherCharacterOverlay){
		mapActivity.getOverlayManager().getCharacterOverlay().setAlpha(FULLY_OPAQUE_APLHA);
		// animate to former player location
    	mapActivity.getMapController().animateTo(mapActivity.getCharactersManager().getMyCharacterLocation(),new Runnable() {	
			public void run() {
		    	try {
					Thread.sleep(TRANSPORT_DURATION);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				// animate to new player location
				mapActivity.getCharactersManager().setMyCharacterLocation(otherCharacterOverlay.getLocation());
				mapActivity.getMapController().animateTo(mapActivity.getCharactersManager().getMyCharacterLocation());   
			}
		});
	}
	
	/**
	 * If dialog is not null, it is hidden at the beginning of the animation and shown back at the end of the animation.
	 * 
	 * @param newLocation
	 * @param dialog
	 */
	public void showNewLocation(GeoPoint newLocation, final Dialog dialog){
		// hide dialog if needed
		if(dialog != null){
			dialog.hide();
		}
    	mapActivity.getMapController().animateTo(newLocation,new Runnable() {	
			public void run() {
				try {
					Thread.sleep(TRANSPORT_DURATION);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				mapActivity.getMapController().animateTo(mapActivity.getCharactersManager().getMyCharacterLocation(), new Runnable() {
					public void run() {
						try {
							Thread.sleep(TRANSPORT_DURATION/2);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
						// show dialog if needed
						if(dialog != null){
							dialog.show();
						}
					}
				});     
			}
		});
	}
}
