package org.playonline.ui.basic.map;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.Projection;

import android.util.Log;
import android.view.GestureDetector.OnGestureListener;
import android.view.MotionEvent;

public class MyOnGestureListener implements OnGestureListener{

	private PlayOnlineMapActivity mapActivity;
	
	
	public MyOnGestureListener(PlayOnlineMapActivity mapActivity){
		this.mapActivity = mapActivity;
	}
	
	
	/**
	 * Notified when a tap occurs with the down MotionEvent that triggered it.
	 */
	public boolean onDown(MotionEvent e) {
		Log.v(getClass().getSimpleName(), "--------------  onDown: "+e.describeContents());
		return false;
	}

	
	/**
	 * The user has performed a down MotionEvent and not performed a move or up yet.
	 */
	public void onShowPress(MotionEvent e) {
		Log.v(getClass().getSimpleName(), "--------------  onShowPress: "+e.describeContents());
	}

	
	/**
	 * Notified when a tap occurs with the up MotionEvent that triggered it.
	 */
	public boolean onSingleTapUp(MotionEvent e) {
		Log.v(getClass().getSimpleName(), "--------------  onSingleTapUp: "+e.describeContents());
		return false;
	}

	
	/**
	 * Notified when a scroll occurs with the initial on down MotionEvent and the current move MotionEvent.
	 */
	public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX,float distanceY) {
		Log.v(getClass().getSimpleName(), "--------------  onScroll: "+e1.describeContents()+" -- "+e2.describeContents()+" -- "+distanceX+" -- "+distanceY);
		return false;
	}

	
	/**
	 * Notified when a long press occurs with the initial on down MotionEvent that triggered it.
	 * Display the context menu of the PlayOnlineMapActivity.
	 */
	public void onLongPress(MotionEvent e) {
		Projection proj = mapActivity.getProjection();
		GeoPoint destination = proj.fromPixels((int)e.getX(), (int)e.getY());
		Log.v(getClass().getSimpleName(), "--------------  onLongPress: "+e.getX()+" -- "+e.getY()+" ---->  "+destination);
		// only if character is alone
		if(!mapActivity.getCharacter().isAttachedToLeader()){
			// display dialog allowing user to travel to the location pressed
			mapActivity.getMapViewDialogsManager().displayTravelDialogToStandardLocation(destination);
		}
	}

	
	/**
	 * Notified of a fling event when it occurs with the initial on down MotionEvent and the matching up MotionEvent.
	 */
	public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX,float velocityY) {
		Log.v(getClass().getSimpleName(), "--------------  onFling: "+e1.describeContents()+" -- "+e2.describeContents()+" -- "+velocityX+" -- "+velocityY);
		return false;
	}

}
