package org.playonline.ui.basic.map;


import org.playonline.network.client.custom.SecondUGASPClient;
import org.playonline.network.client.http.UGASPClientHttp;
import org.playonline.ui.basic.PlayerProfileTabActivity;
import org.playonline.ui.basic.R;
import org.playonline.ui.basic.db.DBAdapter;
import org.playonline.ui.basic.dialog.InGameDialogFrame;
import org.playonline.ui.basic.dialog.MapViewDialogsManager;
import org.playonline.ui.basic.dialog.NewElementsAlertDialog;
import org.playonline.ui.basic.interaction.CharactersManager;
import org.playonline.ui.basic.interaction.GroupManager;
import org.playonline.ui.basic.interaction.InGameProgress;
import org.playonline.ui.basic.map.handler.MapViewHandler;
import org.playonline.ui.basic.map.handler.MapViewMessage;
import org.playonline.ui.basic.map.overlay.OverlayManager;
import org.playonline.ui.basic.map.overlay.PlayOnlineLocationOverlay;
import org.playonline.ui.basic.profile.Character;
import org.playonline.ui.basic.profile.TransportActivity;

import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.database.Cursor;
import android.os.Bundle;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemSelectedListener;
import android.widget.ArrayAdapter;
import android.widget.ImageButton;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Projection;


public class PlayOnlineMapActivity extends MapActivity implements OnItemSelectedListener
{	

	public static final String			GTALK_COMPONENT_NAME 			= "com.google.android.talk/.SigningInActivity";
	public static final int				IVENTORY_REQUEST_CODE 			= 1;	

	public static final int 			CREATE_INSTANCE_PROGRESS_DIALOG = -10;
	public static final int 			JOIN_INSTANCE_PROGRESS_DIALOG 	= -20;

	private static final int			ACCURATE_ZOOM_LEVEL				= 11;
	
	private static final long			TRANSPORT_THREAD_PERIOD			= 2000;//ms

	private 	ArrayAdapter<String> 	arrayAdapter;
	private 	Spinner 				spinner;
	private 	TextView				transportPointsTextView;
	private	 	MapView 				mapView;
	private 	MapController 			mapController;
	private 	OverlayManager 			overlayManager;
	private 	MapViewDialogsManager	mapViewDialogsManager;
	private 	NewElementsAlertDialog	newElementsAlertDialog;
	private 	MapAnimation 			mapAnimation;
	private 	InGameDialogFrame 		dialogFrame;

	protected 	SecondUGASPClient		uGaspClient;
	private 	Thread					uGaspThread;
	private 	DBAdapter				dbAdapter;
	private 	ProgressDialog 			progressDialog;
	private		MapViewHandler			mapViewHandler;

	private 	Character				character;
	private		CharactersManager		charactersManager;
	private		GroupManager 			groupManager;
	
	private volatile boolean stopThreadCount;

	
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		// temporary, until save is not available
		super.onCreate(savedInstanceState);
		// for HTC Desire HD and Nexus (One and S)
		setContentView(R.layout.map);
		// for Galaxy Tab
		//setContentView(R.layout.map_galaxy_tab_8_9);
		// init map view and map controller
		initMapView();
		mapView.setOnCreateContextMenuListener(this);
		initMapController();
		// instantiate overlay manager
		overlayManager = OverlayManager.getInstance(this);
		// init components and buttons of this map activity
		initSpinner();
		initCurrentLocationButton();
		initPlayerProfileButton();
		initGTalkButton();
		// init DB Adapter
		dbAdapter = DBAdapter.getInstance(this);
		dbAdapter.open();
		// init mapViewDialogsManager
		mapViewDialogsManager = MapViewDialogsManager.getInstance(this);
		// init newElementsAlertDialog
		newElementsAlertDialog = NewElementsAlertDialog.getInstance(this);
		// and mapAnimation
		mapAnimation = MapAnimation.getInstance(this);
		// and uGASP Handler
		mapViewHandler = MapViewHandler.getInstance(this);
		// init character, characters manager and group manager
		character = Character.getInstance();
		charactersManager = CharactersManager.getInstance(this);
		groupManager = GroupManager.getInstance(this);
		// init transport point text view
		transportPointsTextView = (TextView)findViewById(R.id.transportPointsTextFields);
		transportPointsTextView.setText("Points transport : "+character.getTransportPoints());
		initTransportActivityReceiver();
		// for the simulation of transport points increase
		stopThreadCount = false;
	}


	@Override
	protected void onDestroy() {
		super.onDestroy();
		// close DB Adapter
		dbAdapter.close();
		if(uGaspThread != null && uGaspClient != null && uGaspClient.getID().isActorSessionIDValid()){
			// quit (and possibly end) Application Instance
			uGaspClient.getLobbyMgr().quitApplicationInstance(uGaspClient.getID().getActorSessionID());
		}
		// TODO add commentary for followers and for demonstration
		InGameProgress.missionMayorUnblocked = false;
		TransportActivity.stopTransportCount();
	}

	
	@Override
	protected boolean isRouteDisplayed() {
		return false;
	}

	
	public void onItemSelected(AdapterView<?> parent, View view, int position,long id) {
		GeoPoint center = PlayOnlineGeoPoint.getGeoPointFromName(parent.getItemAtPosition(position).toString());
		if(center != null){
			mapController.animateTo(center);
			mapController.setZoom(ACCURATE_ZOOM_LEVEL);
		}else{
			Toast.makeText(this, "No GeoPoint matches with " +parent.getItemAtPosition(position).toString(), Toast.LENGTH_LONG).show(); 
		}
	}


	public void onNothingSelected(AdapterView<?> parent) {}


	/**********************************************************************************************************/
	/***************************************	Getters		***************************************************/
	/**********************************************************************************************************/
	
	
	public MapController getMapController(){
		return mapController;
	}
	
	
	public MapView getMapView(){
		return mapView;
	}
	
	
	public OverlayManager getOverlayManager(){
		return overlayManager;
	}
	
	
	public Projection getProjection() {
		return mapView.getProjection();
	}


	public NewElementsAlertDialog getNewElementsAlertDialog(){
		return newElementsAlertDialog;
	}


	public MapViewDialogsManager getMapViewDialogsManager() {
		return mapViewDialogsManager;
	}


	public MapAnimation getMapAnimation(){
		return mapAnimation;
	}


	public SecondUGASPClient getUGASPClient(){
		return uGaspClient;
	}
	
	
	public DBAdapter getDBAdapter(){
		return dbAdapter;
	}


	public Character getCharacter() {
		return character;
	}

	
	public CharactersManager getCharactersManager(){
		return charactersManager;
	}
	

	public GroupManager getGroupManager() {
		return groupManager;
	}

	
	public InGameDialogFrame getDialogFrame(){
		return dialogFrame;
	}
	
	
	public MapViewHandler getMapViewHandler(){
		return mapViewHandler;
	}
	
	
	/**********************************************************************************************************/
	/*************************		Initialization of attributes		***************************************/
	/**********************************************************************************************************/


	private void initSpinner(){
		spinner = (Spinner) findViewById(R.id.spinner);
		arrayAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item);
		arrayAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spinner.setAdapter(arrayAdapter);
		spinner.setOnItemSelectedListener(this);
		// insert new place in PlayOnlineGeoPoint's collection
		PlayOnlineGeoPoint.putGeoPoint("Zone d'arrivée", PlayOnlineGeoPoint.BEGINNING_AREA);
		// insert initial location in spinner
		arrayAdapter.add("Zone d'arrivée");

	}


	private void initMapView(){
		// retrieve map view
		mapView = (MapView)findViewById(R.id.myGoogleMap);
		// custom map view
		mapView.setBuiltInZoomControls(true);
		mapView.setSatellite(true);
	}


	private void initMapController(){
		if(mapView != null){
			// customize map
			mapController = mapView.getController();
			mapController.setZoom(ACCURATE_ZOOM_LEVEL);
		}else{
			Log.e(this.getClass().getSimpleName(), "Map view isn't instantiate yet.");
		}
	}


	private void initCurrentLocationButton(){
		final ImageButton button = (ImageButton) findViewById(R.id.buttonLocation);
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				mapController.animateTo(overlayManager.getCharacterOverlay().getLocation());
			}
		});
	}


	private void initPlayerProfileButton(){
		final ImageButton button = (ImageButton) findViewById(R.id.buttonPlayerProfile);
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				Intent intent = new Intent();
				intent.setClass(PlayOnlineMapActivity.this, PlayerProfileTabActivity.class);
				// add transport points value, for Transport Activity
				intent.putExtra(TransportActivity.POINTS_KEY, character.getTransportPoints());
				startActivity(intent);    
			}
		});
	}


	private void initGTalkButton(){
		final ImageButton button = (ImageButton) findViewById(R.id.buttonGTalkMap);
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				Intent intent = new Intent("android.intent.action.MAIN");
				intent.addCategory("android.intent.category.LAUNCHER");
				intent.setComponent(ComponentName.unflattenFromString(GTALK_COMPONENT_NAME));
				startActivity(intent);  
			}
		});
	}


	private void initTransportActivityReceiver() {
		BroadcastReceiver receiver = new BroadcastReceiver() {
			@Override
			public void onReceive(Context context, Intent intent) {
				if(intent != null){
					if(intent.getAction().equals(TransportActivity.POINTS_ACTION)){
						short pointsToSend = intent.getExtras().getShort(TransportActivity.POINTS_KEY);
						setTransportPoints((short)(character.getTransportPoints() - pointsToSend));
						if(character.isAttachedToLeader()){
							// points are sent to leader
							uGaspClient.getDataTransmissionMgr().getRMAIMgr().remoteMethodAsynchronousInvocation(
																	character.getLeaderActorSessionID(),
																	"sendTransportPoints",
																	Short.valueOf(pointsToSend));
						}
					}else if(intent.getAction().equals(TransportActivity.START_THREAD_COUNT_ACTION)){
						startThreadCount();
					}else if(intent.getAction().equals(TransportActivity.STOP_THREAD_COUNT_ACTION)){
						stopThreadCount();
					}
				}
			}
		};
		IntentFilter filter1 = new IntentFilter(TransportActivity.POINTS_ACTION);
		IntentFilter filter2 = new IntentFilter(TransportActivity.START_THREAD_COUNT_ACTION);
		IntentFilter filter3 = new IntentFilter(TransportActivity.STOP_THREAD_COUNT_ACTION);
		registerReceiver(receiver, filter1);
		registerReceiver(receiver, filter2);
		registerReceiver(receiver, filter3);
	}
	
	
	/******************************************************************************/
	/*****************************  Logging Menu **********************************/
	/******************************************************************************/

	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(0, 1000, 0, "Create Instance");
		menu.add(0, 2000, 0, "Join Instance");
		menu.add(0, 3000, 0, "Set Address");
		return super.onCreateOptionsMenu(menu);
	}

	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch (item.getItemId()) {
		case 1000:
			createInstanceAndWait();
			break;

		case 2000:
			joinInstance();
			break;
			
		case 3000:
			mapViewDialogsManager.displayUGASPAddressDialog();
			break;

		default:
			break;
		}
		return super.onOptionsItemSelected(item);
	}


	/*
	 * Handle at the same times uGASP connection dialog and In game dialog (case default)
	 * (non-Javadoc)
	 * @see android.app.Activity#onCreateDialog(int)
	 */
	@Override
	protected Dialog onCreateDialog(int id) {
		// set proper message
		switch (id) {
		case CREATE_INSTANCE_PROGRESS_DIALOG: {
			// instantiate and customize progress dialog
			progressDialog = new ProgressDialog(this);
			progressDialog.setIndeterminate(true);
			progressDialog.setCancelable(true);
			progressDialog.setMessage("Please wait while creating instance...");
			return progressDialog ;
		}
		case JOIN_INSTANCE_PROGRESS_DIALOG:{
			// instantiate and customize progress dialog
			progressDialog = new ProgressDialog(this);
			progressDialog.setIndeterminate(true);
			progressDialog.setCancelable(true);
			progressDialog .setMessage("Please wait while joining instance...");
			return progressDialog ;
		}
		default:
			// other dialogs are in game dialog with bots -- for group members, text views are disabled.
			dialogFrame = InGameDialogFrame.getInstance(this,(short)id,character.isAttachedToLeader());
			return dialogFrame.getDialog();
		}
	}

	private void loadUGASPUrlFromDB(){
		Cursor cursor = dbAdapter.getAttributes();
		startManagingCursor(cursor);
		cursor.moveToFirst();
		int colValueIndex = cursor.getColumnIndex("value");
		try{
			String address = cursor.getString(colValueIndex);
	        UGASPClientHttp.setUGASPUrl("http://"+address+":8080/uGASP");
		}catch(Throwable t){
			Log.e(getClass().getSimpleName(), "Exception on loading of uGASP address in Database",t);
		}
        
	}
	
	
	private void createInstanceAndWait(){
		loadUGASPUrlFromDB();
		character.setName(SecondUGASPClient.OWNER_USERNAME);
		// initialize and launch uGASPClient
		uGaspClient = SecondUGASPClient.getCreator(this);
		uGaspThread = new Thread(uGaspClient);
		uGaspThread.start();
	}

	
	private void joinInstance(){
		loadUGASPUrlFromDB();
		// initialize and launch uGASPClient with a random user name
		String userName = getRandomUsername();
		character.setName(userName);
		uGaspClient = SecondUGASPClient.getSimpleClient(this,userName);
		uGaspThread = new Thread(uGaspClient);
		uGaspThread.start();
	}
	
	
	private String getRandomUsername(){
		long currentTime = System.currentTimeMillis();
		return new String("user_"+(currentTime%1000));
	}


	public void cancelProgressDialog(){
		if(progressDialog != null){
			progressDialog.cancel();
		}
	}

	/******************************************************************************************************/
	/*******************************************  In game Methods *****************************************/
	/******************************************************************************************************/


	/****************************************/
	/***********    Dialogs     *************/
	/****************************************/
	
	
	public void updateDialogFrame(int textViewIndex, short dialogId){
		dialogFrame.updateData(textViewIndex, dialogId);
	}
	
	
	public void cancelDialogFrame(){
		dialogFrame.cancel();
	}
	
	
	/***********************************************************/
	/***********    Places, Areas and Elements     *************/
	/***********************************************************/
	

	/**
	 * A place is designated by a {@link PlayOnlineLocationOverlay}.
	 * 
	 * @param drawableId
	 * @param location
	 * @param title
	 * @param snippet
	 * @param matchingDialogId
	 */
	public void unblockNewPlace(int drawableId, GeoPoint location, String title, String snippet, short matchingDialogId){
		// insert new place in PlayOnlineGeoPoint's collection
		PlayOnlineGeoPoint.putGeoPoint(snippet, location);
		// insert place to spinner
		arrayAdapter.add(snippet);
		// draw overlay
		PlayOnlineLocationOverlay.drawLocationOverlay(this, drawableId, location, title, snippet,matchingDialogId);
	}
	
	/**
	 * An area is not designated by any overlay.
	 * 
	 * @param location
	 * @param title
	 */
	public void unblockNewArea(GeoPoint location, String title){
		// insert new place in PlayOnlineGeoPoint's collection
		PlayOnlineGeoPoint.putGeoPoint(title, location);
		// insert place to spinner
		arrayAdapter.add(title);
	}
	
	
	/***************************************/
	/***********    Points     *************/
	/***************************************/
	
	public void setTransportPoints(short points){
		this.character.setTransportPoints(points);
		this.transportPointsTextView.setText("Points Transport : "+points);
	}
	
	private void startThreadCount(){
		if(!stopThreadCount){
			Runnable run = new Runnable() {
				public void run() {
					while(!stopThreadCount){
						// send message to increments transport points
						Message msg = MapViewMessage.sendTransportPointsMessage((short)1);
						mapViewHandler.sendMessage(msg);
						try {
							Thread.sleep(TRANSPORT_THREAD_PERIOD);
						} catch (InterruptedException e) {
							Log.e(getClass().getSimpleName(), e.getMessage());
						}
					}
					// Immediately set back boolean to false (so it can be started back later)
					stopThreadCount = false;
				}
			};
			Thread thread = new Thread(run);
			thread.start();
		}
	}
	

	private void stopThreadCount(){
		stopThreadCount = true;
	}
	
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		if(requestCode == 1){
			if(resultCode == 8){
				updateDialogFrame(InGameDialogFrame.NO_TEXT_VIEW_TO_HIGHLIGHT, (short)8);
			}
		}
	}
}
