package org.playonline.ui.basic.map.handler;


import org.playonline.ui.basic.map.PlayOnlineMapActivity;

import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.widget.Toast;

public class MapViewHandler extends Handler {

	public 	final static String DEFAULT_BUNDLE_KEY	= "key";
	
	private PlayOnlineMapActivity mapActivity;
	
	private MapViewHandler(PlayOnlineMapActivity mapActivity){
		this.mapActivity = mapActivity;
	}
	
	public static MapViewHandler getInstance(PlayOnlineMapActivity mapActivity){
		return new MapViewHandler(mapActivity);
	}
	
	@Override
	public void handleMessage(Message msg) {
		int textViewIndex,hiddenElementHashCode;
		short dialogId;
		switch (msg.what) {
		case MapViewWhat.SHORT_TOAST_WHAT:
			String shortToast = (String)(msg.getData().getCharSequence(DEFAULT_BUNDLE_KEY));
			Log.d(getClass().getSimpleName(), "Short Toast displayed --> "+shortToast);
			Toast.makeText(mapActivity, shortToast, Toast.LENGTH_SHORT).show();
			break;
			
		case MapViewWhat.LONG_TOAST_WHAT:
			String longToast = (String)(msg.getData().getCharSequence(DEFAULT_BUNDLE_KEY));
			Log.d(getClass().getSimpleName(), "Long Toast displayed --> "+longToast);
			Toast.makeText(mapActivity, longToast, Toast.LENGTH_LONG).show();
			break;
		
		case MapViewWhat.ALERT_WHAT:
			String alert = (String)(msg.getData().getCharSequence(DEFAULT_BUNDLE_KEY));
			Log.d(getClass().getSimpleName(), "Alert displayed --> "+alert);
			mapActivity.getMapViewDialogsManager().displayAlertMessage(alert);
			break;
			
		case MapViewWhat.CREATE_INSTANCE_PROGRESS_WHAT:
			mapActivity.showDialog(PlayOnlineMapActivity.CREATE_INSTANCE_PROGRESS_DIALOG);
			Log.v(getClass().getSimpleName(), "Create Instance progress dialog displayed.");
			break;
		
		case MapViewWhat.JOIN_INSTANCE_PROGRESS_WHAT:
			mapActivity.showDialog(PlayOnlineMapActivity.JOIN_INSTANCE_PROGRESS_DIALOG);
			Log.v(getClass().getSimpleName(), "Join Instance progress dialog displayed.");
			break;
			
		case MapViewWhat.CANCEL_PROGRESS_WHAT:
			mapActivity.cancelProgressDialog();
			Log.v(getClass().getSimpleName(), "Progress dialog cancelled.");
			break;
		
		case MapViewWhat.UPDATE_CHARACTERS_OVERLAY_WHAT:
			mapActivity.getOverlayManager().updateCharactersOverlays();
			Log.v(getClass().getSimpleName(), "Characters Overlays updated.");
			break;
			
		case MapViewWhat.SHOW_DIALOG_WHAT:
			dialogId = msg.getData().getShort(DEFAULT_BUNDLE_KEY);
			mapActivity.showDialog(dialogId);
			Log.v(getClass().getSimpleName(), "Show dialog "+dialogId+" called.");
			break;
		
		case MapViewWhat.UDPATE_DIALOG_FRAME_WHAT:
			textViewIndex = msg.getData().getInt(MapViewMessage.TEXT_VIEW_INDEX_KEY);
			dialogId = msg.getData().getShort(MapViewMessage.DIALOG_FRAME_ID_KEY);
			mapActivity.updateDialogFrame(textViewIndex, dialogId);
			Log.v(getClass().getSimpleName(), "updateDialogFrame "+dialogId+" called.");
			break;
		
		case MapViewWhat.CANCEL_DIALOG_FRAME_WHAT:
			mapActivity.cancelDialogFrame();
			Log.v(getClass().getSimpleName(), "cancelDialogFrame called.");
			break;
			
		case MapViewWhat.ADD_HIDDEN_ELEMENT_OVERLAY_WHAT:
			hiddenElementHashCode = msg.getData().getInt(MapViewMessage.HIDDEN_ELEMENT_HASHCODE_KEY);
			mapActivity.getOverlayManager().addHiddenElementOverlay(hiddenElementHashCode);
			Log.v(getClass().getSimpleName(), "hiddenElementHashCode called.");
			break;
			
		case MapViewWhat.REMOVE_HIDDEN_ELEMENT_OVERLAY_WHAT:
			hiddenElementHashCode = msg.getData().getInt(MapViewMessage.HIDDEN_ELEMENT_HASHCODE_KEY);
			mapActivity.getOverlayManager().removeHiddenElementOverlay(hiddenElementHashCode);
			Log.v(getClass().getSimpleName(), "removeHiddenElementOverlay called.");
			break;
			
		case MapViewWhat.CATCH_HIDDEN_ELEMENT_WHAT:
			String name = (String)(msg.getData().getCharSequence(MapViewMessage.ELEMENT_NAME_KEY));
			String description = (String)(msg.getData().getCharSequence(MapViewMessage.ELEMENT_DESCRIPTION_KEY));			
			int marker  = msg.getData().getInt(MapViewMessage.HIDDEN_ELEMENT_MARKER_KEY);
			hiddenElementHashCode  = msg.getData().getInt(MapViewMessage.HIDDEN_ELEMENT_HASHCODE_KEY);
			mapActivity.getOverlayManager().catchHiddenElement(name, description, marker, hiddenElementHashCode);
			Log.v(getClass().getSimpleName(), "catchHiddenElement called.");
			break;
			
		case MapViewWhat.ANIMATE_TO_MYLOCATION_WHAT:
			mapActivity.getMapController().animateTo(mapActivity.getCharactersManager().getMyCharacterLocation());
			Log.v(getClass().getSimpleName(), "animateTo called.");
			break;
			
		case MapViewWhat.SEND_TRANSPORT_POINTS_WHAT:
			short pointsToSend = msg.getData().getShort(MapViewMessage.SEND_TRANSPORT_POINT_KEY);
			short newTransportPoints = (short)(mapActivity.getCharacter().getTransportPoints() + pointsToSend);
			mapActivity.setTransportPoints(newTransportPoints);
			Log.v(getClass().getSimpleName(), "setTransportPoints called.");
			break;
			
		default:
			break;
		}
	}
	
}
