package org.playonline.ui.basic.map.overlay;

import java.util.ArrayList;

import org.playonline.ui.basic.interaction.InGameProgress;
import org.playonline.ui.basic.map.PlayOnlineMapActivity;

import android.graphics.drawable.Drawable;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.OverlayItem;


/**
 * Define an Overlay, represented by a given drawable at a given GeoPoint.
 * This overlay is attached to an area (by example : "Reunion Island") which matches with its OverlayItem title,
 * and a specific place (by example : "city hall") which matches with its OverlayItem snippet.
 */
public class PlayOnlineLocationOverlay extends ItemizedOverlay<OverlayItem>{
	
	private ArrayList<OverlayItem> 	mOverlays = new ArrayList<OverlayItem>();
	private PlayOnlineMapActivity 	mapActivity;
	private OverlayItem 			mOverlayItem;
	private short					matchingDialogId;
	
	
	private PlayOnlineLocationOverlay(PlayOnlineMapActivity mapActivity, Drawable defaultMarker, GeoPoint location, String title, String snippet, short matchingDialogId) {
		  super(boundCenterBottom(defaultMarker));
		  this.mapActivity = mapActivity;
		  mOverlayItem = new OverlayItem(location, title, snippet);
		  addOverlay(mOverlayItem);
		  this.matchingDialogId = matchingDialogId;
	}
	
	
	public void addOverlay(OverlayItem overlay) {
	    mOverlays.add(overlay);
	    populate();
	}

	@Override
	protected OverlayItem createItem(int i) {
		return mOverlays.get(i);
	}

	@Override
	public int size() {
		return mOverlays.size();
	}

	
	@Override
	protected boolean onTap(int index) {
		// only if character is alone
		if(!mapActivity.getCharacter().isAttachedToLeader()){
			// if player is located on the given overlay
			if(getCenter().equals(mapActivity.getCharactersManager().getMyCharacterLocation())){
				mapActivity.showDialog(matchingDialogId);
				if(mapActivity.getCharactersManager().isMyCharacterLeader()){
					mapActivity.getGroupManager().showDialogToGroup(matchingDialogId);
				}
			}else{
				// Display dialog to go to PlayOnline location
				OverlayItem item = mOverlays.get(index);
				computeSuitableTravelDialog(item);                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     
			}
		}
		return true;
	}
	
	// TODO Should be completed with new location 
	private void computeSuitableTravelDialog(OverlayItem item){
		if(this.matchingDialogId == InGameProgress.mayorFirstDialog){
			mapActivity.getMapViewDialogsManager().displayTravelDialogToPlayOnlineLocation(item.getTitle(), item.getSnippet(),
					// by walk																		
					15,0,
					// by bike
					3,20,
					// by car
					0,60);
		}else if(this.matchingDialogId == InGameProgress.fieldsWorkerFirstDialog){
			mapActivity.getMapViewDialogsManager().displayTravelDialogToPlayOnlineLocation(item.getTitle(), item.getSnippet(),
					// by walk																		
					30,0,
					// by bike
					9,50,
					// by car
					0,200);
		}
	}


	/**
	 * Draw a new PlayOnlineLocationOverlay in a PlayOnlineMapActivity.
	 * 
	 * @param mapActivity 	the PlayOnlineMapActivity where overlay will be drawn.
	 * @param drawableID	the id of the drawable representing the overlay.
	 * @param location		the GeoPoint where the overlay will be drawn on the map.
	 * @param title			the title of the overlay.
	 * @param snippet		the snippet of the overlay.
	 */
	public static void drawLocationOverlay(PlayOnlineMapActivity mapActivity, int drawableID, GeoPoint location, String title, String snippet,short matchingDialogId){
		// retrieve the drawable from resources
		Drawable marker = mapActivity.getResources().getDrawable(drawableID);
		// create a new overlay
		PlayOnlineLocationOverlay locationOverlay = new PlayOnlineLocationOverlay(mapActivity, marker, location, title, snippet, matchingDialogId);
		// add it to mapActivity's overlays
		mapActivity.getOverlayManager().addOverlay(locationOverlay);	
	}
}
