package org.playonline.ui.basic.profile;


import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.GridView;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.List;

import org.playonline.ui.basic.R;
import org.playonline.ui.basic.dialog.InGameDialogFrame;


public class InventoryActivity extends Activity implements OnItemClickListener {

	public static String NEW_ELEMENT_DRAWABLE 			= "drawable";
	public static String NEW_ELEMENT_TITLE 				= "title";
	public static String NEW_ELEMENT_DESCRIPTION		= "description";
	
	private static final List<Element> inventory 		= initInventory();
	private static final short 	DEFAULT_DIALOG_FRAME_ID = -1;
	
	private TextView elementTitle;
	private TextView elementDescription;
	
	private ImageAdapter adapter;
	private short 	currentDialogFrameDataId;
	private int 	currentItemSelected;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// for HTC Desire HD and Nexus (One and S)
		setContentView(R.layout.layout_animation_inventory);
		// for Galaxy Tab
		//setContentView(R.layout.layout_animation_inventory_galaxy_tab_8_9);
		initUseItemButton();
		Bundle extras = getIntent().getExtras(); 
		if(extras != null){
			currentDialogFrameDataId = extras.getShort(InGameDialogFrame.CURRENT_DIALOG_FRAME_DATA_ID_KEY);
		}else{
			currentDialogFrameDataId = DEFAULT_DIALOG_FRAME_ID;
		}

		GridView grid = (GridView) findViewById(R.id.grid);
		this.adapter= new ImageAdapter(this);
		grid.setAdapter(adapter);
		grid.setOnItemClickListener(this);
		
		elementTitle = (TextView)findViewById(R.id.elementTitle);
		elementTitle.setText("Titre");
		elementDescription = (TextView)findViewById(R.id.elementDescription);
		elementDescription.setText("Description");
	}
	
	private void initUseItemButton(){
		final ImageButton button = (ImageButton) findViewById(R.id.useItemButton);
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				computeItem(currentItemSelected);
			}
		});
	}
	

	/**
	 * Add an element to the inventory.
	 * 
	 * @param title
	 * @param description
	 * @param drawableId
	 */
	public static void addElement(String title, String description, int drawableId){
		Element element = new Element(title, description, drawableId);
		inventory.add(element);
	}
	
	private static List<Element> initInventory() {
		ArrayList<Element> list = new ArrayList<Element>();
		list.add(new Element("Essaim endomagé", 
							"Essaim trouvé près de la plantation de canne à sucre.\n"+
							"A faire analyser.",
							R.drawable.nid_de_frelons));
		list.add(new Element("Branche", "Branche d'arbre trouvée sur le lieu d'arrivée.",R.drawable.branche));
		// for HTC Desire HD and Nexus (One and S)
		list.add(new Element("Bouteille d'azote", "Utilisation non connue.",R.drawable.bouteille_azote));
		// for Galaxy Tab
		/*list.add(new Element("Autorisation spéciale", 
				"Carte confiée par un responsable territorial.\n"+
				"Peut s'avérer utile pour avoir la confiance des habitants.", 
				R.drawable.carte_magnetique));*/
		return list;
	}
	
	
	public void onItemClick(AdapterView<?> parent, View view, int position,long id) {
		elementTitle.setText(inventory.get(position).getTitle());
		elementDescription.setText(inventory.get(position).getDescription());
		currentItemSelected = position;
	}
	
	
	private void computeItem(int itemPosition){
		if(currentDialogFrameDataId == 7 && inventory.get(itemPosition).getTitle().equals("Autorisation spéciale")){
			inventory.remove(itemPosition);
			setResult(8);
			finish();
		}
	}

	public class ImageAdapter extends BaseAdapter {
		
		private Context context;
		
		public ImageAdapter(Context context) {
			this.context = context;
		}

		public int getCount() {
			return inventory.size();
		}

		public Object getItem(int position) {
			return inventory.get(position);
		}

		public long getItemId(int position) {
			return 0;
		}

		public View getView(int position, View convertView, ViewGroup parent) {
			ImageView imageView;
			if (convertView == null) {
				imageView = new ImageView(context);
				imageView.setLayoutParams(new GridView.LayoutParams(100, 100));
				imageView.setAdjustViewBounds(false);
				imageView.setScaleType(ImageView.ScaleType.FIT_CENTER);
			} else {
				imageView = (ImageView) convertView;
			}
			imageView.setImageResource(inventory.get(position).getDrawableId());
			return imageView;
		}

	}
}

