package org.playonline.ui.basic.profile;

import org.playonline.ui.basic.R;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.ImageButton;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import android.widget.SeekBar;
import android.widget.SeekBar.OnSeekBarChangeListener;
import android.widget.TextView;

public class TransportActivity extends Activity implements OnSeekBarChangeListener, OnCheckedChangeListener{



	public static final String 	POINTS_ACTION				= "pointsKeyAction";
	public static final String 	START_THREAD_COUNT_ACTION 	= "startThreadAction";
	public static final String 	STOP_THREAD_COUNT_ACTION 	= "stopThreadAction";
	public static final String 	POINTS_KEY 					= "pointsKey";
	private  static boolean threadCountLaunched				= false;
	private  static boolean transportChecked				= false;
	
	private short 		availableTransportPoints;
	private TextView 	availableTransportPointsTextView;
	private TextView 	pointsToAddTextView;
	private SeekBar		seekBar;
	private	RadioGroup	radioGroup;
	private RadioButton	transportRadioButton;
	
	public static void stopTransportCount(){
		threadCountLaunched = false;
		transportChecked = false;
	}
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		// retrieve data from PlayOnlineTabActivity
		Bundle extras = getIntent().getExtras(); 
		if(extras != null){
			availableTransportPoints = extras.getShort(POINTS_KEY);
		}else{
			availableTransportPoints = 0;
		}
		setContentView(R.layout.transport);
		// available points
		availableTransportPointsTextView = (TextView)findViewById(R.id.availablePointsTextView);
		availableTransportPointsTextView.setText("Points transport : "+availableTransportPoints);
		// points to add
		pointsToAddTextView = (TextView)findViewById(R.id.pointsToAddTextView);
		seekBar = (SeekBar) findViewById(R.id.seekBar);
		seekBar.setOnSeekBarChangeListener(this);
		// adjust seek bar
		seekBar.setMax(availableTransportPoints);
		// button to send points
		initAddPointsImageButton();
		// init radio group
		initRadioGroup();
	}

	
	public void onProgressChanged(SeekBar seekBar, int progress,boolean fromUser) {
		String text = "Envoyer "+progress+" points transport au leader ?";
		pointsToAddTextView.setText(text);
	}


	public void onStartTrackingTouch(SeekBar seekBar) {}


	public void onStopTrackingTouch(SeekBar seekBar) {}
	
	
	private void initAddPointsImageButton() {
		ImageButton button = (ImageButton) findViewById(R.id.addPointsButton);
		button.setOnClickListener(new View.OnClickListener() {
			public void onClick(View v) {
				// update available points
				int pointsToSend = seekBar.getProgress();
				availableTransportPoints = (short)(availableTransportPoints - pointsToSend);
				// adjust seek bar
				seekBar.setMax(availableTransportPoints);
				// Send to platform
				broadcastPointsToSend((short)pointsToSend);
				// finish activity
				finish();
			}
		});
	}
	
	
	private void initRadioGroup() {
		radioGroup = (RadioGroup) findViewById(R.id.radioGroup);
		radioGroup.setOnCheckedChangeListener(this);
		transportRadioButton = (RadioButton) findViewById(R.id.transportRadioButton);
		if(transportChecked){
			transportRadioButton.setChecked(true);
		}
	}
	
	
	private void broadcastPointsToSend(short pointsToSend){
		Intent intent = new Intent(POINTS_ACTION);
		intent.putExtra(POINTS_KEY, pointsToSend);
		sendBroadcast(intent);
	}


	public void onCheckedChanged(RadioGroup group, int checkedId) {
		if(checkedId == transportRadioButton.getId()){
			if(!threadCountLaunched){
				threadCountLaunched = true;
				transportChecked = true;
				Intent intent = new Intent(START_THREAD_COUNT_ACTION);
				sendBroadcast(intent);
			}
		}else{
			threadCountLaunched = false;
			transportChecked = false;
			Intent intent = new Intent(STOP_THREAD_COUNT_ACTION);
			sendBroadcast(intent);
		}
	}
	
}
